(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     51551,       1876]*)
(*NotebookOutlinePosition[     52636,       1911]*)
(*  CellTagsIndexPosition[     52592,       1907]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["6. Lists II:  Iteration and List Handlers", "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision: January 26 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We introduce iterators in some generality, and then we continue our \
exploration of list-based functions, exploring ways to construct a list \
(typically of numbers) and ways to deconstruct a list into various pieces.  \
Some of this material you have seen before, and if you fully understand it, \
feel free to move on.  Finally, we outline an analogy between ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code and language grammar, studying how some functions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " affect the operation of others."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[
  "     FullForm[]\n     Part[]\n     Take[]\n     Drop[]\n     Outer[]\n     \
TableForm[]\n     MapThread[]\n     . ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["operator",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["\n     Dot[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Iterators"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Iterators are lists which instruct functions such as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " on how many iterations, or steps, a certain operation should be repeated. \
 ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If an iterator is of the form \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{number}", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", then the function is instructed to do the operation \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" times.  For example, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{30}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" as iterator means to repeat the operation 30 times.  We have already \
used this in our ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" command:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[r,{5}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "If an iterator is of the form \"",
  StyleBox["{variable, number}", "Input",
    FontWeight->"Plain"],
  "\", then the function is instructed to do the operation \"",
  StyleBox["number", "Input",
    FontWeight->"Plain"],
  "\" times, where the first time \"",
  StyleBox["variable", "Input",
    FontWeight->"Plain"],
  "\" will be ",
  StyleBox["1", "Input",
    FontWeight->"Plain"],
  ", the second time ",
  StyleBox["2", "Input",
    FontWeight->"Plain"],
  ", and so on, up until \"",
  StyleBox["number", "Input",
    FontWeight->"Plain"],
  "\"."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[r, {r,5}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If an iterator is of the form \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{variable, number1, number2}", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", then the function is instructed to do the operation \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" times, where the first time \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["variable", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" will be the value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", the second time \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" plus ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[", and so on, until \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["variable", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" reaches \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\".  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["minimum",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" value; \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["maximum ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["value.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[r, {r,2,6}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If an iterator is of the form \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{variable, number1, number2, number3}", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", then the function is instructed to do the operation \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" times, where the first time \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["variable", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" will be the value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", the second time \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" plus \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number3", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", the third time \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" plus 2 times \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number3", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" and so on, until \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["variable", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" reaches \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\".  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["minimum ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["value; \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["maximum",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" value; \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number3", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["step",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" value.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[r, {r,2,6,0.5}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  "\" expressions need not be numbers but larger expressions if necessary, \
and it is possible to have several iterators in a row.  In this case it is \
not true that the first iterator executes, finishes, and then the next one \
executes, and so on.  Instead the iterators are nested.  For instance, in the \
two-iterator case, the first iterator starts at its first value, the second \
iterates through all of its valued, and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["then",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " the first iterator steps to its next value, the second iterates through \
all of its values, and so on.  Here is an example:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[Pi^r E^s,{r,2,6},{s,1,5}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[expression,{a,amin,amax},{b,bmin,bmax}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" operates in the same fashion that nested \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" functions do, with the iterators reversed; for example, the previous \
expression is the same as ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[Table[Pi^r E^s,{s,1,5}],{r,2,6}]", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["List Generators (List, Table, Range, Operations)"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["First, recall that the functional form of a list (given by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["FullForm[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\") is simply \"Li",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["st[x, y, z, .... ]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"; thus, the most naive way to form a list is simply to write out each of \
its elements.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["List[1,2,3,4]", "Input",
  AspectRatioFixed->True],

Cell["{1,2,3,4}//FullForm", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Another way to create a list is to form a table of values, using an \
iterator."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[x,{x,0,1,.2}]", "Input",
  AspectRatioFixed->True],

Cell["Table[x^2,{x,0,1,.2}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A third way is to use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[]", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Plain"],
  StyleBox["\", a primitive form of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" which simply gives the value of the iterator (with no variable \
specified).  Notice that for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" braces are unnecessary.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["xlist = Range[0,1,.2]", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Finally, many operations which usually apply to numbers can be applied to \
list, and typically the operation is applied to each element of the list.  \
Experiment with other operations!"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["ylist = xlist^2", "Input",
  AspectRatioFixed->True],

Cell["zlist = xlist + 5", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Extracting from List (Part, Take, Drop)"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "We've already seen one way to get a n element or portion of a list, using \
double brackets [[]].  Recall that the functional form of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["[[]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[list, ... ]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" are the \"core\" of list extraction operators.  In fact, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" can be defined using \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Let's look at some examples.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Let's say we want the first \"n\" elements of a given list.  In the first \
argument of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" we must place the list from which we want elements extracted, and in \
the second argument of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" we must place a list of the positions, or indices, we want extracted \
from the list.  To form a list with the indices from 1 through \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we will use the expression \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[n]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Here's a function definition of our operation:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["myTake[list_, n_] := Part[list, Range[n]]", "Input",
  AspectRatioFixed->True],

Cell["\<\
Clear[a,b,c,d,e,f];
myTake[{a,b,c,d,e,f},3]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Actually, though, there's a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " function which does exactly what we've defined.  This function is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[":  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[list,number]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" gives the first \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" elements of the list.  Similarly, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[list,{number1,number2}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" gives the elements from position \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" to position \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\".  An interesting variation is that if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" is negative, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" gives that many elements from the end.  (Note that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["myTake[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" does not have these additional features; some additional programming is \
necessary to make \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["myTake[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" fully match \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Take[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".)",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Take[{a,b,c,d,e,f},3]", "Input",
  AspectRatioFixed->True],

Cell["Take[{a,b,c,d,e,f},-3]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A related function is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[list,n]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" returns \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with its first \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" elements dropped (omitted); if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is negative, the last \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" elements are dropped; and if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is of the form \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{number1, number2}", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", then the elements from position \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" to position \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["' are dropped.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Drop[{a,b,c,d,e,f},2]", "Input",
  AspectRatioFixed->True],

Cell["Drop[{a,b,c,d,e,f},-2]", "Input",
  AspectRatioFixed->True],

Cell["Drop[{a,b,c,d,e,f},{3,5}]", "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData[
"Example of List-Based Programming:  Finding Midpoints of Lists"], 
  "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Let's use these operations to consider a larger problem, that of finding \
all of the midpoints of a list of numbers.  A midpoint of two numbers \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["y",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is the number \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(x+y)/2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"; it's the average of the two numbers.  Finding all of the midpoints of \
a list means, for us, finding midpoints of all consecutive numbers in the \
list, i.e., the midpoint of the first and second numbers, the midpoint of the \
second and third, and so on.  This problem is a good one for list-based \
programming, because it affords us an opportunity to use list operations.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We'll begin with a list \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["xlist",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["xlist = {0., 0.2, 0.4, 0.6, 0.8, 1.}", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "What we want to do is add consecutive numbers and then divide by 2, but \
how might we do this only with list operations?  Well, we need a list of the \
numbers we want to add, and we need a list of the numbers we want added to \
the first list.  More precisely, we want a list of the left-hand addends, and \
a list of the right-hand addends.  We can do this with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  First, the left-hand addends:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Drop[xlist,-1]", "Input",
  AspectRatioFixed->True],

Cell[TextData["Then, the right-hand addends:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Drop[xlist,1]", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Now we'll simply add the two lists and divide by 2!  Note that lists are \
added element-by- element, and the divide-by-2 operation is applied to each \
element of the resulting list."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["(Drop[xlist,-1] + Drop[xlist,1]) / 2", "Input",
  AspectRatioFixed->True],

Cell[TextData["Let's make a function out of this:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
midpoints[list_] := 
   (Drop[list,-1] + Drop[list,1]) / 2\
\>", "Input",
  AspectRatioFixed->True],

Cell["midpoints[xlist]", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Another Example:  Riemann Sums"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "For another example, let's consider how we might evaluate some Riemann \
sums of the function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  The algorithm is somewhat complicated, but nothing we cannot \
accomplish using the operations we have covered.  A Riemann sum for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with \"n\" divisions between x-values \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" (with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Plain"],
  StyleBox["\"<\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\") is formed by finding \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" numbers equally spaced between \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", evaluating \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" at each of them, multiplying these values by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", and adding up the results.  As always, we approach the problem by \
taking small steps first.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["To find \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" numbers equally spaced between \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", we use a result from our mathematics class that the numbers \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a+(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a+2(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a+3(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", up to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a+(n-1)(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" are equally spaced from \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"---being \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" apart, each time.  How do we generate these numbers?  We can use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", with the appropriate iterator.  Note that the beginning value is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", the ending value is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a+(n-1)(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and the step value is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  To aid us in working out the code, let's look at a specific example \
first, say with 1 for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", 2 for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and 10 for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  We want, then, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[1, 1+(9)(2-1)/10, (2-1)/10]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Range[1,1+(9)(2-1)/10,(2-1)/10]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now we want to evaluate \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" at each of them.  Using list-based operations, we can instruct ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" to raise each value to the second power:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Range[1,1+(9)(2-1)/10,(2-1)/10] ^ 2", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now we must multiply each of these values by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", or, in our case, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(2-1)/10",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  We'll place the multiplication operation at the end of the line:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Range[1,1+(9)(2-1)/10,(2-1)/10] ^ 2 * (2-1)/10", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Finally, we must use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[Plus, ]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to add up each of the values in the list.  (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" requires an iterator, which we do not have; similarly, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[Sum, ]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" would be incorrect.)",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Apply[Plus,
   Range[1,1+(9)(2-1)/10,(2-1)/10] ^ 2 
   * (2-1)/10
]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Our code is correct.  Looking back, however, we find that we can improve \
the speed of the code, using the distributive property of multiplication, as \
follows.  Instead of multiplying each element of the list by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and then adding them up---which takes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" multiplications followed by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"-1 additions---we could add the elements up first and then multiply by \
\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(b-a)/n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  We then save \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"-1 of those multiplications:  adding the elements up first and \
multiplying once takes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"-1 additions and 1 multiplication.  Let's rewrite our code to take \
advantage of this improved method:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Apply[Plus,
   Range[1,1+(9)(2-1)/10,(2-1)/10] ^ 2 
] * (2-1)/10\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Leaving the definition of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["square[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we can form the rest of the code into a function:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[mySquareSum]
mySquareSum[a_,b_,n_] := Apply[
   Plus,
   Range[
      a,
      a+(n-1)(b-a)/n,
      (b-a)/n
   ] ^ 2 
] * (b-a)/n\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData["Let's check that it works:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["mySquareSum[1,2,10]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Finally, we point out that one can accomplish the same operations using \
\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Sum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" instead of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Range[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Consider the similarities and differences between \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["mySquareSum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and the following function, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["newSquareSum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[newSquareSum]
newSquareSum[a_,b_,n_]:=
\t(b-a)/n * Sum[i^2,{i,a,a+(n-1)(b-a)/n,(b-a)/n}]\
\>", "Input",
  AspectRatioFixed->True],

Cell["newSquareSum[1,2,10]", "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Grammar and Examples of Adverbs (Inner, Outer, Apply, MapThread)"], "Section",\

  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "A good analogy which might be made between ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code and language grammar is as follows.  A \"noun\" is anything defined \
with an equals sign and no formal parameters (expressions followed by \
underscores inside brackets); a \"verb\" is anything defined with \
colon-equals and with formal parameters; and an \"adverb\" is a function \
which takes another function and applies it in a certain fashion.  In this \
section we want to consider some \"adverbs\"."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Suppose we have a function which takes two arguments and we wish to apply \
it to two lists in the following fashion:  we want to use the function with \
the first argument taken from the first list and the second taken from the \
second list, and we want to do so for all possible combinations.  The adverb \
which applies a function in this fashion is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Outer[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Let's make two lists and watch it work.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["sideList = Range[.16,.20,.01]", "Input",
  AspectRatioFixed->True],

Cell["topList = Range[10,15]", "Input",
  AspectRatioFixed->True],

Cell["Outer[Times, sideList, topList]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Sometimes it's useful to show the output of a list of lists in a tabular \
form.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Expression//TableForm",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" or \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["TableForm[expression]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" will accomplish this for us.  Note that each innermost list is a row \
(not a column) of the table.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["%//TableForm", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note:  It's best not so save the lists with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["TableForm[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", since this isn't the easiest way for ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" to work with them.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["What if we don't want a function so complicated as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Outer[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", but simply want to apply a function to all elements of the list \
together, such as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["+",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["?  Here \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is the function (adverb) we want.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["xlist", "Input",
  AspectRatioFixed->True],

Cell["Apply[Plus,xlist]", "Input",
  AspectRatioFixed->True],

Cell["Apply[Times,xlist]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["What if we want a function like \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Outer[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", except that we don't want all possible combinations of taking a first \
argument from a first list and a second from a second?  Let's say we want to \
take arguments, one at a time, from each list, say using the first elements \
of each list together, then the second, and so forth.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MapThread[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" is the function (adverb) we want:  it takes a function of two or more \
arguments and matches up arguments from a list of lists.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["First we'll define an averaging function."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["ave2[a_, b_] := (a+b)/2", "Input",
  AspectRatioFixed->True],

Cell["ave2[3,7]", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"And now we'll use this function to average grades, say for students, where \
the first list is the scores on the first test and the second list is the \
scores on the second list (in the same order)."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["MapThread[ave2,{{3,4,5},{7,8,9}}]", "Input",
  AspectRatioFixed->True],

Cell["xlist", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "This average function gives us a new way to find our midpoints; we simply \
use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MapThread[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["ave2[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" and our two truncated lists.  First we'll need to make a list of the \
two lists:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["twolists = {Drop[xlist,-1],Drop[xlist,1]}", "Input",
  AspectRatioFixed->True],

Cell[TextData["And then here we go:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["MapThread[ave2,twolists]", "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["A dot product"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "A certain operation in mathematics is to take two lists, multiply \
corresponding entries, and add up all of the products.  We will do this with \
the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" commands we know.  First, though, let's note that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" already knows how to do this, with the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" operator:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[a,b,c];
{1,2,3}.{a,b,c}  (* Built-in dot product *)\
\>", "Input",
  AspectRatioFixed->True],

Cell["Dot[{1,2,3},{a,b,c}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "First we want to multiply corresponding entries.  This is accomplished \
just by using \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MapThread[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Times[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["MapThread[Times,{{1,2,3},{a,b,c}}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Then we want to add up all the elements. Recalling that \"",
  StyleBox["%", "Input",
    FontWeight->"Plain"],
  "\" denotes the result of the previous expression, we try the following:"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Apply[Plus,%]", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"The code appears to work, so we'll put the two lines together, programming \
bottom-up:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Apply[Plus,
   MapThread[Times,{{1,2,3},{a,b,c}}]
]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Finally, satisfied with our code thus far, we form the code into a \
function:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
myDot[list1_,list2_] := 
\tApply[ Plus, MapThread[ Times, {list1,list2} ] ]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Note how we had to put brackets around the two lists to make a \
list-of-lists for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MapThread[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{520, 509},
WindowMargins->{{76, Automatic}, {15, Automatic}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003809T?o>old</PW30fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@4b=X0000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 106, 2, 136, "Title",
  Evaluatable->False],
Cell[1840, 55, 99, 2, 26, "SmallText",
  Evaluatable->False],
Cell[1942, 59, 649, 14, 98, "Text",
  Evaluatable->False],
Cell[2594, 75, 684, 23, 152, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3303, 102, 86, 2, 50, "Section",
  Evaluatable->False],
Cell[3392, 106, 961, 36, 47, "Text",
  Evaluatable->False],
Cell[4356, 144, 1052, 36, 38, "Text",
  Evaluatable->False],
Cell[5411, 182, 55, 1, 22, "Input"],
Cell[5469, 185, 635, 22, 38, "Text",
  Evaluatable->False],
Cell[6107, 209, 58, 1, 22, "Input"],
Cell[6168, 212, 2517, 90, 51, "Text",
  Evaluatable->False],
Cell[8688, 304, 60, 1, 22, "Input"],
Cell[8751, 307, 3282, 118, 64, "Text",
  Evaluatable->False],
Cell[12036, 427, 64, 1, 22, "Input"],
Cell[12103, 430, 1046, 28, 77, "Text",
  Evaluatable->False],
Cell[13152, 460, 74, 1, 22, "Input"],
Cell[13229, 463, 681, 22, 38, "Text",
  Evaluatable->False],
Cell[13913, 487, 81, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[14031, 493, 125, 2, 30, "Section",
  Evaluatable->False],
Cell[14159, 497, 668, 22, 38, "Text",
  Evaluatable->False],
Cell[14830, 521, 56, 1, 22, "Input"],
Cell[14889, 524, 62, 1, 22, "Input"],
Cell[14954, 527, 154, 4, 25, "Text",
  Evaluatable->False],
Cell[15111, 533, 62, 1, 22, "Input"],
Cell[15176, 536, 64, 1, 22, "Input"],
Cell[15243, 539, 864, 30, 38, "Text",
  Evaluatable->False],
Cell[16110, 571, 64, 1, 22, "Input"],
Cell[16177, 574, 261, 5, 38, "Text",
  Evaluatable->False],
Cell[16441, 581, 58, 1, 22, "Input"],
Cell[16502, 584, 60, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[16599, 590, 116, 2, 30, "Section",
  Evaluatable->False],
Cell[16718, 594, 1794, 64, 51, "Text",
  Evaluatable->False],
Cell[18515, 660, 1236, 40, 64, "Text",
  Evaluatable->False],
Cell[19754, 702, 84, 1, 22, "Input"],
Cell[19841, 705, 94, 4, 30, "Input"],
Cell[19938, 711, 2692, 93, 90, "Text",
  Evaluatable->False],
Cell[22633, 806, 64, 1, 22, "Input"],
Cell[22700, 809, 65, 1, 22, "Input"],
Cell[22768, 812, 2101, 76, 51, "Text",
  Evaluatable->False],
Cell[24872, 890, 64, 1, 22, "Input"],
Cell[24939, 893, 65, 1, 22, "Input"],
Cell[25007, 896, 68, 1, 22, "Input"],

Cell[CellGroupData[{
Cell[25100, 901, 146, 4, 42, "Subsection",
  Evaluatable->False],
Cell[25249, 907, 1200, 34, 77, "Text",
  Evaluatable->False],
Cell[26452, 943, 341, 13, 25, "Text",
  Evaluatable->False],
Cell[26796, 958, 79, 1, 22, "Input"],
Cell[26878, 961, 715, 18, 64, "Text",
  Evaluatable->False],
Cell[27596, 981, 57, 1, 22, "Input"],
Cell[27656, 984, 103, 2, 25, "Text",
  Evaluatable->False],
Cell[27762, 988, 56, 1, 22, "Input"],
Cell[27821, 991, 258, 5, 38, "Text",
  Evaluatable->False],
Cell[28082, 998, 79, 1, 22, "Input"],
Cell[28164, 1001, 108, 2, 25, "Text",
  Evaluatable->False],
Cell[28275, 1005, 109, 4, 30, "Input"],
Cell[28387, 1011, 59, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[28483, 1017, 110, 2, 42, "Subsection",
  Evaluatable->False],
Cell[28596, 1021, 2546, 90, 77, "Text",
  Evaluatable->False],
Cell[31145, 1113, 3921, 144, 90, "Text",
  Evaluatable->False],
Cell[35069, 1259, 74, 1, 22, "Input"],
Cell[35146, 1262, 616, 21, 38, "Text",
  Evaluatable->False],
Cell[35765, 1285, 78, 1, 22, "Input"],
Cell[35846, 1288, 625, 21, 38, "Text",
  Evaluatable->False],
Cell[36474, 1311, 89, 1, 22, "Input"],
Cell[36566, 1314, 816, 27, 38, "Text",
  Evaluatable->False],
Cell[37385, 1343, 118, 6, 50, "Input"],
Cell[37506, 1351, 1737, 57, 77, "Text",
  Evaluatable->False],
Cell[39246, 1410, 115, 5, 40, "Input"],
Cell[39364, 1417, 394, 13, 25, "Text",
  Evaluatable->False],
Cell[39761, 1432, 187, 11, 100, "Input"],
Cell[39951, 1445, 100, 2, 25, "Text",
  Evaluatable->False],
Cell[40054, 1449, 62, 1, 22, "Input"],
Cell[40119, 1452, 1200, 43, 38, "Text",
  Evaluatable->False],
Cell[41322, 1497, 145, 5, 40, "Input"],
Cell[41470, 1504, 63, 1, 22, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[41582, 1511, 144, 4, 48, "Section",
  Evaluatable->False],
Cell[41729, 1517, 594, 12, 64, "Text",
  Evaluatable->False],
Cell[42326, 1531, 715, 18, 64, "Text",
  Evaluatable->False],
Cell[43044, 1551, 72, 1, 22, "Input"],
Cell[43119, 1554, 65, 1, 22, "Input"],
Cell[43187, 1557, 74, 1, 22, "Input"],
Cell[43264, 1560, 710, 24, 51, "Text",
  Evaluatable->False],
Cell[43977, 1586, 55, 1, 22, "Input"],
Cell[44035, 1589, 594, 20, 38, "Text",
  Evaluatable->False],
Cell[44632, 1611, 1004, 36, 38, "Text",
  Evaluatable->False],
Cell[45639, 1649, 48, 1, 22, "Input"],
Cell[45690, 1652, 60, 1, 22, "Input"],
Cell[45753, 1655, 61, 1, 22, "Input"],
Cell[45817, 1658, 947, 26, 64, "Text",
  Evaluatable->False],
Cell[46767, 1686, 115, 2, 25, "Text",
  Evaluatable->False],
Cell[46885, 1690, 66, 1, 22, "Input"],
Cell[46954, 1693, 52, 1, 22, "Input"],
Cell[47009, 1696, 274, 5, 38, "Text",
  Evaluatable->False],
Cell[47286, 1703, 76, 1, 22, "Input"],
Cell[47365, 1706, 48, 1, 22, "Input"],
Cell[47416, 1709, 667, 24, 38, "Text",
  Evaluatable->False],
Cell[48086, 1735, 84, 1, 22, "Input"],
Cell[48173, 1738, 94, 2, 25, "Text",
  Evaluatable->False],
Cell[48270, 1742, 67, 1, 22, "Input"],

Cell[CellGroupData[{
Cell[48362, 1747, 93, 2, 42, "Subsection",
  Evaluatable->False],
Cell[48458, 1751, 909, 30, 51, "Text",
  Evaluatable->False],
Cell[49370, 1783, 108, 4, 30, "Input"],
Cell[49481, 1789, 63, 1, 22, "Input"],
Cell[49547, 1792, 591, 22, 25, "Text",
  Evaluatable->False],
Cell[50141, 1816, 77, 1, 22, "Input"],
Cell[50221, 1819, 266, 7, 38, "Text",
  Evaluatable->False],
Cell[50490, 1828, 56, 1, 22, "Input"],
Cell[50549, 1831, 162, 4, 25, "Text",
  Evaluatable->False],
Cell[50714, 1837, 102, 5, 40, "Input"],
Cell[50819, 1844, 153, 4, 25, "Text",
  Evaluatable->False],
Cell[50975, 1850, 126, 4, 30, "Input"],
Cell[51104, 1856, 407, 15, 25, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

