(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     52909,       1861]*)
(*NotebookOutlinePosition[     53997,       1896]*)
(*  CellTagsIndexPosition[     53953,       1892]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["10. Anonymous Functions", "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision: February 9 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this section we introduce anonymous functions, explore their \
similarity to inline functions in compiled languages, and show how anonymous \
functions can simplify code for certain operations.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["anonymous functions",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; Function, #,  &",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\n     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[
  "First[]\n     Last[]\n     Select[]\n     NumberQ[], IntegerQ[], EvenQ[], \
OddQ[], PrimeQ[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Anonymous Functions", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["In the previous section we considered defining functions using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Module", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["s and the colon-equals (",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["SetDelayed",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[") operator \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[":", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["=", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Plain"],
  StyleBox[
  "\".  When we did so, we gave the new function and each of its parameters a \
name.  This form might be called the procedural form, or long form, of a \
function definition.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " provides another form which eliminates the need to name the function or \
its parameters; instead, the expression itself, without names, \"is\" the \
function definition, hence the term \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["anonymous function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Bold"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Why would we want to use expressions which ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["represent",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" functions instead of simply",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" defining",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " them with names?  There are several reasons.  First, we may only need to \
use the function one particular time, in which case we need not name it to \
use it again.  Second, the representations of anonymous functions provide a \
simple shortcut to defining functions, and while some anonymous functions are \
extremely difficult to read, the simpler ones are just as easy to read as the \
equivalent named functions defined using colon-equals.  Finally, we introduce \
\"anonymous functions\" because they have a cousin in many compiled \
languages, which we describe as follows.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Inline functions"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Computers do not actually execute the source code written in any language; \
in one way or another, the source code must be converted into ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["machine code",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  "---the only code the computer understands---and then executed.  In \
interpreted languages such as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", this conversion happens \"on-the-fly\" in the sense that only when we \
ask to execute a cell (and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["every",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " time we ask to execute a cell) is the code converted. In contrast, for \
compiled languages such as C, Fortran, or Pascal, the source code is passed \
through a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["compiler",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " which performs the conversion, leaving the final machine code in a file \
which can be executed repeatedly without reconversion.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "During the compilation process, the compiler of a standard computer \
language reads the source code several times, collecting the names of \
variables, assigning them storage locations, checking for errors, and so on. \
Typically one \"pass\" over the source code is devoted to expanding out \
so-called \"inline functions\", and these are cousins to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s \"anonymous functions\". Let's examine the difference between the way \
an inline function and a normal function is embedded in machine code by \
looking at how a compiler might convert some ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code into a machine language.  (Of course, this machine language is \
intended to be representative only; no computer exists which executes these \
machine instructions.)"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Suppose the code is the following:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["f[x_] := x^2 + x + 5; a = f[10]+f[20]+f[30]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " were not declared inline, then the machine code might look like this:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
(1) Copy \"10\" to \"x\" inside function \"f\"
(2) Go to function \"f\" (step 13), execute its instructions, and return
(3) Copy result of function \"f\" to temporary variable \"t\"
(4) Copy \"20\" to \"x\" inside function \"f\"
(5) Go to function \"f\" (step 13), execute its instructions, and return
(6) Copy result of function \"f\" to temporary variable \"u\"
(7) Copy \"30\" to \"x\" inside function \"f\"
(8) Go to function \"f\" (step 13), execute its instructions and return
(9) Copy result of function \"f\" to variable \"a\"
(10) Add \"a\" and \"u\" and store the result in \"a\"
(11) Add \"a\" and \"t\" and store the result in \"a\"
(12) Stop
(13) [Beginning of function \"f\"] Square \"x\" and store in temporary \
variable \"v\"
(14) Add \"v\" and \"x\" and store the result in temporary variable \"v\"
(15) Add \"v\" and \"5\" and store the result in our result variable \"result\
\"
(16) Return to original (\"calling\") instruction\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Note that the main code (steps 1--12) jumps down to step 13 three separate \
times to execute the function, and that  each of these times it copies \
beforehand a value into the variable ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " of the function and, after the function executes, copies the result of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " back to a variable.  These jumps, together with the copy operations \
before and after the jumps, are called ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["function calls",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now suppose ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " were declared inline.  Then the machine code might instead look like \
this:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
(1) Square \"10\" and store value in temporary variable \"t\"
(2) Add \"t\" and \"10\" and store the result in \"t\"
(3) Add \"t\" and \"5\" and store the result in \"t\"
(4) Square \"20\" and store value in temporary variable \"u\"
(5) Add \"u\" and \"20\" and store the result in \"u\"
(6) Add \"u\" and \"5\" and store the result in \"u\"
(7) Square \"30\" and store value in temporary variable \"v\"
(8) Add \"v\" and \"30\" and store the result in \"v\"
(9) Add \"v\" and \"5\" and store the result in \"v\"
(10) Add \"u\" and \"v\" and store the result in \"a\"
(11) Add \"t\" and \"a\" and store the result in \"a\"
(13) Stop\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here we find instead ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["no ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  "function calls; instead the instructions of the function are embedded in \
the main code, one set (1-3, 4-6, 7-9) for each reference (",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[10]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[20]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[30]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[") of the function.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"What tradeoffs are present in the choice to use inline or normal functions?  \
First and foremost, programs using inline functions execute faster.  This \
increase in speed is due to the fact that the program does not have to spend \
time copying values back and forth between the main code and the function's \
own separate variables.  The drawback is that inline functions frequently \
require more lines of code---more memory devoted to the program itself.  We \
do not see this drawback in the case above, since the function itself needed \
only a few instructions, but suppose the function needed 200 machine \
instructions to accomplish its task and let's count how many machine \
instructions would be necessary to cover each of the three references to the \
function.  We would need a few instructions---a few copies, and the jump \
instruction---for each reference of the function, together with one instance \
of all of the instructions for the function, placed at the end.  At three \
instructions per function call, plus 200 instructions for the function at the \
end, we find we need 209 instructions.  However, with inline functions, each \
of the 200 instructions would need to appear inside the main code every time \
the function were referenced, yielding fully 600 instructions devoted to the \
function references.  Thus we have a tradeoff between speed of execution of \
the program and memory requirements for the program."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["To summarize, an ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["inline function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " is a function which is not implemented in the standard fashion, using one \
set of instructions for the function, together with one function call for \
each reference.  Instead, when the compiler finds an inline function \
referenced in the source code, it immediately inserts the instructions of the \
function into the program itself---hence the term \"inline\"---and does not \
insert a \"function call\".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Syntax of Anonymous Functions", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Consider the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["square", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" function which returns the square of its argument.",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[square];
square[x_] := x^2;\
\>", "Input",
  AspectRatioFixed->True],

Cell["Map[ square, {1,2,3,4,5} ]", "Input"],

Cell[TextData[{
  StyleBox["Basically, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["square", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" is a function of one argument (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[")which returns the square of that argument (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  Cell[BoxData[
      \(TraditionalForm\`x\^2\)]],
  ")",
  StyleBox[".  We could more literally translate this prose definition of ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["square", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" into ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[" as follows.",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[square];
square = Function[ {x}, x^2 ];\
\>", "Input"],

Cell["Map[ square, {1,2,3,4,5} ]", "Input"],

Cell[TextData[{
  StyleBox["That is, we assign to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["square", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" a ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" object which performs the appropriate calculation.  This ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " object is an entity in its own right, for it must have been constructed \
before it was assigned to the variable ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["square", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  ".  As such, we might have used it without giving it a name.  For example,",
    
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Map[ Function[ {x}, x^2 ], {1,2,3,4,5} ]", "Input"],

Cell[TextData[{
  StyleBox["The expression \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Function[ {x}, x^2 ]", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\" is an example of an ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["anonymous function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  "An anonymous functions has no name, it's just \"the function that \
\[Ellipsis].\"  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["The general form for this is ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function[", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["parameter-list, body", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["]", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "There is another, more succinct, notation for anonymous functions.  In \
this notation, the named parameter(s) are replaced by \
\"slot(s)\"\[LongDash]place holders for parameters.  To convert a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" to this notation, extract the body of the ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " and replace every occurrence of the name of the parameter with the symbol \
\"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["#", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  "\" (if there are more than one parameter, replace occurrences of the first \
parameter by the symbol \"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["#1", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", occurrences of the second parameter by the symbol \"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["#2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["\", etc.).  Finally, wrap the modified ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " body inside parentheses (to be safe) and append the \"anonymous function \
operator,\" ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["&", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[", to the expression.  ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Applying this technique to the anonymous squaring function, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Function[ {x}, x^2 ]", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[", produces:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["(#^2)&", "Input",
  AspectRatioFixed->True],

Cell["Map[ (#^2)&, {1,2,3,4,5} ]", "Input"],

Cell[TextData[{
  StyleBox["Check out the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["FullForm", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  " of this expression, and you'll see it is an abbreviation, of sorts, for a \
",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" expression.",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["FullForm[(#^2)&]", "Input"],

Cell[TextData[{
  StyleBox[
  "Yes, the anonymous function is somewhat harder to read.  For an example of \
a case where using an anonymous function simplifies code, consider the \
following situation.  We wish to iterate the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Exp[-x]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " over and over again beginning with the value 1.  One solution is the \
following:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
newExp[x_] := Exp[-x];
Nest[newExp,1.,100]\
\>", "Input",
  AspectRatioFixed->True],

Cell["\<\
It is irritating that we have to go to the trouble of defining a \
separate function in order to use such a simple operation.   The anonymous \
function solution is much simpler:\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Nest[Function[{x}, Exp[-x]],1.,100]", "Input",
  AspectRatioFixed->True],

Cell["Or, ", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Nest[Exp[-#]&,1.,100]", "Input",
  AspectRatioFixed->True],

Cell[TextData["(Note we didn't need the parentheses this time.)"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"What follows are several examples of bottom-up programming, coupled with a \
judicious use of anonymous functions."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Example 1: Replication"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "By now you should appreciate the idea of building up a function slowly, \
starting with the simple things and gradually adding more layers.  Let's try \
that idea again now.  We want to write a function called ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " which should take a list of integers and return a list where each number \
n (in the list) has been replicated n times.  For example, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[{3,2,4,2}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" should return ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{3,3,3,2,2,4,4,4,4,2,2}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ".  As a first simple step, how can we replicate a single number, say 3? \
We'll begin with the following:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[f]\nf[n_]:=Table[n,{n}]\nf[3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now we want to replicate each number in the list ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{3,2,4,2}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", so we'll map ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" onto the list.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Map[f,{3,2,4,2}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Now all we need to do is use ",
  StyleBox["Flatten[]", "Input",
    FontWeight->"Plain"],
  " on this list. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "(It always helps to remember that,  when trying to write a new function in \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", we should do three things:  (1) clear any old definition; (2) make the \
new definition; and (3) test the new definition.)"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[replicate]\nreplicate[list_]:=Flatten[ Map[f,list] ]\n\
replicate[{3,2,4,2}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["One problem with our ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" function is that it relies on the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " which we defined earlier.  By now you've probably experienced the problem \
of trying to execute a function that relied on something you'd not yet \
defined.  (Imagine trying to find ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["roster[[4]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" without having first executed ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["roster={....}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[". )  But even if we do remember to define ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", before trying ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", we have to remember not to redefine ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " as something else in a different problem.  For example, if the next \
problem in this lab asked us to plot ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f(x)=x+Sin[x]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" we might casually enter: ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[f]\nf[x_]= x+Sin[x]\nPlot[f[x],{x,-2Pi,2Pi}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" will still run, but it will not run correctly.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["replicate[{3,2,4,2}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Clearly using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" in the definition for ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " is a very dangerous method.  We now know two solutions to this problem:  \
(1) use a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Module",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[": create a function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" inside a Module that only the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " knows about, and (2) use an anonymous function: use the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" without ever giving it a name.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Module Solution"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[replicate,f]\n\nreplicate[list_]:= \n  Module[{f},\n\t   \
f[n_]:=Table[n,{n}];\n\t   Flatten[ Map[f, list] ]\n\t ]\n\n\
replicate[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["One advantage of the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Module[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" command is that the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" is defined in the same place as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", so we don't have to worry about forgetting to execute the definition for \
",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" before using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  The great advantage of using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Module",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", however, is that our definition of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" doesn't show up outside of the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" function.  Let's check.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["?f"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We can now use ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" elsewhere, as in ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["f[x_]= x+Sin[x]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["replicate[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" still works just fine.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["replicate[{3,2}]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Anonymous Function Solution", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We want to replace the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " we defined up above with an anonymous function.  Recall that we had \
defined ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[n_]:=Table[ n, {n}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[". ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Function", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[", this converts to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Function[{n}, Table[ n, {n}] ]", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Function[{n}, Table[n,{n}]] [3]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Alternatively, following our rules for converting the definition into an \
expression with no named parameters, we get  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[ #, {#}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  So the anonymous function is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[ #,{#}]&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" or, more safely,  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(Table[ #,{#}])&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Table[#,{#}]& [3]", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that the 3 gets substituted into ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["each",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" of the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["#",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["'s, and the resulting ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[3,{3}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " is executed.  Now we can map this anonymous function onto a list just as \
we did the named function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[replicate]\nreplicate[list_]:=\n\t\tFlatten[ Map[ Table[#,{#}]&, list] \
]\nreplicate[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Using an anonymous function gave us a shorter segment of code than using a \
",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Module",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", and this segment will also run a little bit faster. To see this, let's \
time how long it takes for each to run 10 times:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[replicate1,replicate2]

replicate1[list_]:= 
  Module[{f},
\t   f[n_]:=Table[n,{n}];
\t   Flatten[ Map[f, list] ]
\t ];
replicate2[list_]:=
\t\tFlatten[ Map[ Table[#,{#}]&, list] ];\
\>", "Input",
  AspectRatioFixed->True],

Cell["Timing[Do[replicate1[{1,2,3}],{10}]]", "Input",
  AspectRatioFixed->True],

Cell["Timing[Do[replicate2[{1,2,3}],{10}]]", "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Example 2: Selecting some {x,y} pairs (First, Last, Select)"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"The following code gives a framed plot of 50 random points in the unit \
square in the first quadrant."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
pairs=Table[{Random[],Random[]},{50}];
ListPlot[pairs,
\t Frame->True,
\t AspectRatio->Automatic,
\t PlotRange->{{0,1},{0,1}}];\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We would like to write a function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["aboveLine[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " that will select from the points above those that are above the line ",
    Evaluatable->False,
    AspectRatioFixed->True],
  Cell[BoxData[
      \(TraditionalForm\`y = x\)]],
  StyleBox[
  ".  In other words, we would like to decide, for each point, whether the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["y",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" value is greater than the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " value---this is what \"above the line\" means mathematically---and then \
choose only those points from the first random list.  We'll use some \
functions ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["First[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Last[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" which operate on lists as might be expected:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["aboveLineOne[point_]:= Last[point]>First[point]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["aboveLineOne[{.2,.4}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["aboveLineOne[{.6,.2}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "It looks as if we've solved the problem of deciding whether a point is \
above the line y=x, but rather than defining the function out in the open, \
let's place this function inside a Module.  Clear the current definition of ",
    
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["aboveLineOne[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" before starting.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[aboveLineOne]"], "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Module Solution"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We will use the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Select[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" function.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Select[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " takes two arguments, the first a list from which we want to extract \
certain elements, the second a function which, for each element of the list, \
will give a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["True",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["False",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" answer to whether we want the element.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[aboveLine]\n\naboveLine[points_]:=\n  Module[{aboveLineOne},\n\t   \
aboveLineOne[point_]:=\n\t      Last[point]>First[point];\n\t   \
Select[points, aboveLineOne]\n\t ]\n\nabovePts=aboveLine[pairs];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Let's plot these points and see whether our code is correct."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"ListPlot[abovePts,\n\t Frame->True,\n\t PlotRange->{{0,1},{0,1}},\n\t \
Axes->None,\n\t AspectRatio->Automatic];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "All the points do seem to be above ",
  Cell[BoxData[
      \(TraditionalForm\`y = x\)]],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Anonymous Function Solution", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We want an anonymous version of the function: \n\t",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["aboveLineOne[point_] := Last[point] > First[point]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\nNow ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["point",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" is the only parameter, so we use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["#\"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" in place of it, which yields an expressed anonymous function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox["Last[#]>First[#])&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  Let's try it out:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["(Last[#]>First[#])& [{3,4}]", "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Now we use the function to select the points above the line."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[aboveLine]
aboveLine[points_]:=
\t Select[ points, (Last[#]>First[#])& ]
abovePts=aboveLine[pairs];\
\>", "Input",
  AspectRatioFixed->True],

Cell["\<\
ListPlot[abovePts,
\t Frame->True,
\t PlotRange->{{0,1},{0,1}},
\t Axes->None,
\t AspectRatio->Automatic];\
\>", "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Example 3: Selecting multiples of k from a list (Q functions)"], "Subsection",\

  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Suppose that we want to know which of the following integers are multiples \
of some integer we specify, such as, say, 3."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"numbers={23,5,3,345,67,345,245,4,2,2,5,36,3,42,2,436}"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "How do we know if an Integer is a multiple of another Integer?  We might \
ask a simpler question:  how do we know if a value is an Integer?  Can we ask \
",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["?  Yes.  There are boolean functions ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["NumberQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["IntegerQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["EvenQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["OddQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and even ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["PrimeQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", which simply return True or False depending on the value of the \
argument.  The \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Q",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in the names is a shorthand for the word \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Q",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["uery\".  Let's try a few:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["IntegerQ[3.5]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["NumberQ[2/3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["OddQ[2]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["PrimeQ[373587883]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"We return to our original question in light of these boolean functions.  To \
know whether or not one Integer is a multiple of another, we divide and ask \
whether the result is an Integer."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Module Solution"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We incorporate ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["multK[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" into the definition of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["multiples[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", using the value of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["k",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" which is an argument of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["multiples[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[multiples,multK]\nmultiples[numbers_,k_]:=\n  Module[{multK},\n\t   \
multK[x_]:=IntegerQ[x/k];\n\t   Select[numbers, multK ]\n\t ]\n\
multiples[numbers,3]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Anonymous Function Solution", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We replace ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["multK[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" with the anonymous (nameless) equivalent, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["IntegerQ[#/k] &",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[multiples,multK]\nmultiples[numbers_,k_]:=\n\t\t\
Select[numbers,IntegerQ[#/k]&]\nmultiples[numbers,3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Here the anonymous function has only one argument from elements in the \
list ",
  StyleBox["numbers", "Input",
    FontWeight->"Plain"],
  "; ",
  StyleBox["k", "Input",
    FontWeight->"Plain"],
  " appears as an argument to the function ",
  StyleBox["multiples", "Input",
    FontWeight->"Plain"],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Use of Anonymous Functions", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "The examples above suggest that anonymous functions are best employed when \
(a) the operation(s) to be performed are simply expressed, (b) the operations \
use relatively few parameters, and (c) when we need to use a function which \
requires, as one of its parameters, the name of a function, in which place we \
may place the anonymous function.  Particularly appropriate functions which \
take functions as parameters are ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Select[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Map[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Macintosh",
ScreenRectangle->{{0, 1152}, {0, 850}},
WindowToolbars->{},
WindowSize->{554, 415},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003809T?o>old</PW30fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400A7M;U0000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 88, 2, 98, "Title",
  Evaluatable->False],
Cell[1830, 55, 99, 2, 26, "SmallText",
  Evaluatable->False],
Cell[1932, 59, 268, 6, 46, "Text",
  Evaluatable->False],
Cell[2203, 67, 956, 34, 96, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3184, 105, 86, 2, 50, "Section",
  Evaluatable->False],
Cell[3273, 109, 1614, 52, 98, "Text",
  Evaluatable->False],
Cell[4890, 163, 1137, 28, 128, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[6052, 195, 96, 2, 46, "Subsection",
  Evaluatable->False],
Cell[6151, 199, 1469, 45, 134, "Text",
  Evaluatable->False],
Cell[7623, 246, 913, 18, 146, "Text",
  Evaluatable->False],
Cell[8539, 266, 108, 2, 30, "Text",
  Evaluatable->False],
Cell[8650, 270, 281, 10, 30, "Text",
  Evaluatable->False],
Cell[8934, 282, 384, 14, 30, "Text",
  Evaluatable->False],
Cell[9321, 298, 1020, 21, 270, "Text",
  Evaluatable->False],
Cell[10344, 321, 1040, 33, 80, "Text",
  Evaluatable->False],
Cell[11387, 356, 400, 15, 30, "Text",
  Evaluatable->False],
Cell[11790, 373, 704, 15, 206, "Text",
  Evaluatable->False],
Cell[12497, 390, 990, 36, 48, "Text",
  Evaluatable->False],
Cell[13490, 428, 1518, 21, 238, "Text",
  Evaluatable->False],
Cell[15011, 451, 749, 19, 96, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[15797, 475, 99, 2, 30, "Subsection",
  Evaluatable->False],
Cell[15899, 479, 407, 14, 30, "Text",
  Evaluatable->False],
Cell[16309, 495, 84, 4, 42, "Input"],
Cell[16396, 501, 43, 0, 27, "Input"],
Cell[16442, 503, 1296, 47, 50, "Text",
  Evaluatable->False],
Cell[17741, 552, 70, 3, 42, "Input"],
Cell[17814, 557, 43, 0, 27, "Input"],
Cell[17860, 559, 1231, 42, 62, "Text",
  Evaluatable->False],
Cell[19094, 603, 57, 0, 27, "Input"],
Cell[19154, 605, 1363, 49, 65, "Text",
  Evaluatable->False],
Cell[20520, 656, 2174, 71, 126, "Text",
  Evaluatable->False],
Cell[22697, 729, 403, 13, 30, "Text",
  Evaluatable->False],
Cell[23103, 744, 49, 1, 27, "Input"],
Cell[23155, 747, 43, 0, 27, "Input"],
Cell[23201, 749, 654, 24, 46, "Text",
  Evaluatable->False],
Cell[23858, 775, 33, 0, 27, "Input"],
Cell[23894, 777, 604, 18, 62, "Text",
  Evaluatable->False],
Cell[24501, 797, 93, 4, 42, "Input"],
Cell[24597, 803, 250, 6, 46, "Text",
  Evaluatable->False],
Cell[24850, 811, 78, 1, 27, "Input"],
Cell[24931, 814, 68, 2, 30, "Text",
  Evaluatable->False],
Cell[25002, 818, 64, 1, 27, "Input"],
Cell[25069, 821, 122, 2, 30, "Text",
  Evaluatable->False],
Cell[25194, 825, 189, 4, 46, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[25420, 834, 102, 2, 30, "Subsection",
  Evaluatable->False],
Cell[25525, 838, 1159, 34, 110, "Text",
  Evaluatable->False],
Cell[26687, 874, 88, 1, 57, "Input"],
Cell[26778, 877, 559, 20, 30, "Text",
  Evaluatable->False],
Cell[27340, 899, 69, 1, 27, "Input"],
Cell[27412, 902, 189, 7, 30, "Text",
  Evaluatable->False],
Cell[27604, 911, 339, 9, 48, "Text",
  Evaluatable->False],
Cell[27946, 922, 136, 3, 57, "Input"],
Cell[28085, 927, 2028, 67, 110, "Text",
  Evaluatable->False],
Cell[30116, 996, 103, 1, 57, "Input"],
Cell[30222, 999, 367, 13, 30, "Text",
  Evaluatable->False],
Cell[30592, 1014, 73, 1, 27, "Input"],
Cell[30668, 1017, 1445, 51, 78, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[32138, 1072, 98, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[32239, 1076, 199, 4, 147, "Input"],
Cell[32441, 1082, 1803, 64, 78, "Text",
  Evaluatable->False],
Cell[34247, 1148, 55, 1, 27, "Input"],
Cell[34305, 1151, 341, 13, 30, "Text",
  Evaluatable->False],
Cell[34649, 1166, 68, 1, 27, "Input"],
Cell[34720, 1169, 345, 13, 30, "Text",
  Evaluatable->False],
Cell[35068, 1184, 69, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[35174, 1190, 100, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[35277, 1194, 599, 22, 46, "Text",
  Evaluatable->False],
Cell[35879, 1218, 573, 21, 30, "Text",
  Evaluatable->False],
Cell[36455, 1241, 74, 1, 27, "Input"],
Cell[36532, 1244, 850, 29, 62, "Text",
  Evaluatable->False],
Cell[37385, 1275, 60, 1, 27, "Input"],
Cell[37448, 1278, 986, 36, 48, "Text",
  Evaluatable->False],
Cell[38437, 1316, 154, 3, 72, "Input"],
Cell[38594, 1321, 517, 17, 62, "Text",
  Evaluatable->False],
Cell[39114, 1340, 238, 11, 147, "Input"],
Cell[39355, 1353, 79, 1, 27, "Input"],
Cell[39437, 1356, 79, 1, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[39565, 1363, 140, 3, 30, "Subsection",
  Evaluatable->False],
Cell[39708, 1368, 177, 4, 30, "Text",
  Evaluatable->False],
Cell[39888, 1374, 178, 7, 87, "Input"],
Cell[40069, 1383, 1517, 51, 98, "Text",
  Evaluatable->False],
Cell[41589, 1436, 100, 1, 27, "Input"],
Cell[41692, 1439, 74, 1, 27, "Input"],
Cell[41769, 1442, 74, 1, 27, "Input"],
Cell[41846, 1445, 570, 17, 62, "Text",
  Evaluatable->False],
Cell[42419, 1464, 72, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[42516, 1469, 98, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[42617, 1473, 1055, 37, 62, "Text",
  Evaluatable->False],
Cell[43675, 1512, 259, 4, 162, "Input"],
Cell[43937, 1518, 135, 3, 30, "Text",
  Evaluatable->False],
Cell[44075, 1523, 166, 3, 87, "Input"],
Cell[44244, 1528, 174, 7, 30, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[44455, 1540, 100, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[44558, 1544, 1147, 38, 78, "Text",
  Evaluatable->False],
Cell[45708, 1584, 70, 1, 27, "Input"],
Cell[45781, 1587, 135, 3, 30, "Text",
  Evaluatable->False],
Cell[45919, 1592, 156, 6, 72, "Input"],
Cell[46078, 1600, 157, 7, 87, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[46284, 1613, 144, 4, 30, "Subsection",
  Evaluatable->False],
Cell[46431, 1619, 196, 4, 46, "Text",
  Evaluatable->False],
Cell[46630, 1625, 107, 2, 27, "Input"],
Cell[46740, 1629, 1858, 67, 96, "Text",
  Evaluatable->False],
Cell[48601, 1698, 66, 1, 27, "Input"],
Cell[48670, 1701, 65, 1, 27, "Input"],
Cell[48738, 1704, 60, 1, 27, "Input"],
Cell[48801, 1707, 70, 1, 27, "Input"],
Cell[48874, 1710, 264, 5, 46, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[49163, 1719, 98, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[49264, 1723, 910, 34, 46, "Text",
  Evaluatable->False],
Cell[50177, 1759, 215, 4, 117, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[50429, 1768, 100, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[50532, 1772, 545, 20, 30, "Text",
  Evaluatable->False],
Cell[51080, 1794, 162, 3, 72, "Input"],
Cell[51245, 1799, 387, 14, 46, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[51681, 1819, 96, 2, 30, "Subsection",
  Evaluatable->False],
Cell[51780, 1823, 1089, 33, 94, "Text",
  Evaluatable->False]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

