(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     53490,       1828]*)
(*NotebookOutlinePosition[     54521,       1861]*)
(*  CellTagsIndexPosition[     54477,       1857]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[
"Chapter IV.  Pattern Matching in Symbolic Programming"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["11. Pattern Matching and Conditional Evaluation"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision:  February 18 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In this section we introduce several ways in which ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s pattern matching capabilities permit better coding of functions.  \
Pattern-matching is a feature found primarily in symbolic programming \
languages such as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", but not in procedural or object-oriented languages such as C, C++, or \
Pascal.  The most basic sort of pattern matching we consider is data type \
checking for parameters, followed by general boolean parameter tests (and \
their variants, conditions) and then more structured patterns, with which we \
can assign parameter names to parts of a parameter.  During this section we \
also treat ways in which code can be written to be readable and \
user-friendly, informing the user of invalid parameters."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["parameter",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["_?",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["testQ\n     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["Print[]\n     Trace[]\n     ??",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\n     ::usage=\n     Positive[]\n     /;\n     \\;",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData[
"Data Type Checking and Other Boolean Parameter Tests"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Data Type Checking"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Most functions require, or at least \"expect\", certain data types for the \
values they receive as parameters.  That is, a function written to accept a \
list as its parameter may not work properly if it is invoked with a string as \
its parameter instead.  For instance, suppose we are developing a function to \
produce Pascal's Triangle, the first few rows of which are the following."], 
  "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[OutputFormData[
"\<\
TableForm[{{\" \", \" \", \" \", \" \", 1, \" \", \" \", \" \", \" \", \
\"\\
 
     \"}, {\" \", \" \", \" \", 1, \" \", 1, \" \", \" \", \" \", \" \"}, 
 
   {\" \", \" \", 1, \" \", 2, \" \", 1, \" \", \" \", \" \"}, 
 
   {\" \", 1, \" \", 3, \" \", 3, \" \", 1, \" \", \" \"}, 
 
   {1, \" \", 4, \" \", 6, \" \", 4, \" \", 1, \" \"}, 
 
   {\" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \"}, 
 
   {\" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \"}, 
 
   {\" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \"}, 
 
   {\" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \"}, 
 
   {\" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \", \" \"}}]\
\
\>", "\<\
                1                    

            1       1                

        1       2       1            

    1       3       3       1        

1       4       6       4       1\
\>"], "Output",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"One operation we might require in the production of the triangle is the \
appending and prepending of \"1\"'s; each row can be derived from the row \
above by taking sums of successive entries and then prepending and appending \
a \"1\".  Such a function might be defined as follows:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[addOnes]\naddOnes[list_] := Append[Prepend[list,1],1]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Our function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["addOnes[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" works well if the actual parameter substituted for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" in the function is, in fact, a list; otherwise, disastrous results \
occur.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["addOnes[\"disaster\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "A good programming principle is that functions should \"catch\" these \
errors before they can happen;  functions should determine first whether the \
input values of the parameters are valid, and only then proceed to use the \
values.  If the input values are not suitable for the function, a message \
should be returned to the user, explaining that fact.  We can implement this \
principle using data type checking, where we specify that a definition of a \
function should be executed only when certain conditions on the data types of \
input values of the parameters hold true.  \n\nTo have ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " automatically check a type of a parameter, insert the name of the data \
type after the underscore in the function definition.  For example, instead \
of defining \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[x_]:=x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we can insist that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" have the definition \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" for integers only by executing \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[x_Integer] := x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\". Similarly, to insist that the parameter \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" above be of type List, we use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list_List",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" instead of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list_",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in the function definition.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[addOnes]\naddOnes[list_List] := Append[Prepend[list,1],1]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["addOnes[{4,6,4}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["addOnes[\"disaster\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Notice that when ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " determines that the data type does not match the required type, it simply \
returns the executed expression back, unevaluated.  We can instruct ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " to allow several different data types by writing separate definitions for \
the function with each of the specified data types.  For example, we define \
below several different definitions for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["addOnes[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", depending on the data type of the input:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[addOnes]\naddOnes[list_List] := Append[Prepend[list,1],1]\n\
addOnes[s_String] := StringJoin[\"1\",s,\"1\"]\naddOnes[i_Integer] := 1 + i + \
1"], "Input",
  AspectRatioFixed->True],

Cell[TextData["addOnes[\"dog\"]\naddOnes[{4,6,4}]\naddOnes[50]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Boolean Parameter Tests; Error Messages (Print)"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We may, however, want to check our data more generally.  For instance, \
might we want to define a function which works on all numbers (including, \
say, Integers, Rationals, and Reals), but not on strings?  Might we want to \
define a function which only works on positive integers?  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides a way for us to apply any boolean test to each parameter, \
instructing ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " that the given definition of the function should be used only if each of \
the boolean tests returns a True.  The syntax is to place a boolean \
function's name, with a question mark prepended, after the underscore of the \
corresponding parameter."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[addOnes]\naddOnes[list_List] := Append[Prepend[list,1],1]\n\
addOnes[i_?NumberQ] := 1 + i + 1\n\
addOnes[s_?((StringQ[#]&&StringLength[#]>3)&)] :=\n\t\t\tStringJoin[s,\" and \
so on\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"addOnes[{4,6,4}]\naddOnes[\"dog\"]\naddOnes[\"Dogs and cats\"]\n\
addOnes[45.4]\naddOnes[23]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Recall that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " has a whole array of \"Q\" functions which perform boolean tests on their \
parameters; we have used \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["NumberQ[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" above.  Also, note the use of an anonymous boolean function to check \
whether or not the parameter is a String and if its length is greater than 3. \
Remember to put parentheses around an entire anonymous function definition, \
including the final ampersand (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"), when using it as a parameter check.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Using boolean tests, we can alert the user to the fact that the parameter \
values are in error.  To do so, write an extra definition which tests whether \
the data is invalid and uses a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Print[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" statement in the definition to print an error message.  For example, to \
alert the user that only lists are appropriate for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["addOnes[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we might expand our definition as follows:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[addOnes]\naddOnes[list_List] := Append[Prepend[list,1],1]\n\
addOnes[list_?(!ListQ[#]&)] := \n\t\t\tPrint[\"Parameter not a list!\"]"], 
  "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that the function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Print[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" displays its parameter but returns nothing; in this way we prevent our \
function from passing some string back to an expression that used it.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["addOnes[{4,6,4}]\naddOnes[4]"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Using Specific and General Definitions Together"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Specific First, General After "], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We saw above that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " can store several definitions for the same function.  In fact, every time \
we execute a definition for a function, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" adds this definition to its list, unless we have used \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Clear[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to erase the previous definitions.  How, then, does ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " decide which definition to use, if several definitions are appropriate?  \
The answer, roughly, is that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " attempts to decide which of the valid definitions places the least \
general, or most specific, constraints on its parameters, and uses that \
definition.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " will not always be able to determine which definition is \"most \
specific\", but in most of the cases we will consider, its internal rules for \
deciding specificity will work correctly.  Let us consider how we can use \
this versatility of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" to our advantage.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Suppose we must implement a peculiar function, one which squares its input \
unless that input is a 3, in which case it returns an 8.  Using our boolean \
tests above, we would likely write the following code."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[f];\nf[x_?((#==3)&)] = 8;\nf[x_?((#!=3)&)] = x^2;"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Table[{i,f[i]},{i,1,5}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We can accomplish the same operations by instructing ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" that the general definition is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[x_]:=x^2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", while a more specific rule is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[3]=8",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Note that we do not append an underscore to the 3:  if we do not wish \
to allow any value for the parameter except for the value specified, we do \
not append an underscore.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[f];\nf[3]=8;\nf[x_] := x^2;"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Table[{i,f[i]},{i,1,5}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Let's look at what definitions ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" has stored for our function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["f[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["?f"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now let us recode the function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["addOnes[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to accept only lists, returning an error message otherwise.  We can do \
so much more cleanly than we could before:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Clear[addOnes]
addOnes[list_List] := Append[Prepend[list,1],1]
addOnes[list_] := 
\tPrint[\"Parameter \\\"\",list,\"\\\" is not a list!\"]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData["addOnes[{2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["addOnes[1]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["(Note the interesting use of the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Print[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" function: we are instructing it to print out the parameter \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" with two strings on either side.  The first string ends with a \
quotation mark inside, indicated by a special code---a slash before a \
quotation mark---and the second string begins with the same quotation mark.)  \
",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Application:  Using Multiple Definitions for Recursion (Trace)"], 
  "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Multiple definitions are often used when implementing the method of \
recursion, where a function, during execution, executes itself again, with \
other parameters. Recall our example of the recursive solution to \
implementing a factorial function:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[fac]\nfac[1] = 1 ;\nfac[n_] := n fac[n-1]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["fac[5]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Notice that, during evaluation, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[5]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" becomes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["5 fac[4]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which becomes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["5 4 fac[3]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (multiplication understood where spaces occur), which, after a few more \
iterations, becomes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["5 4 3 2 fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", at which point the specific definition is used, and we have \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["5 4 3 2 1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" simplifies to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["120",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "A function which is helpful in determining how a user-defined function \
operates is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" displays the progress of execution of a function by returning a list of \
the successive evaluations and simplifications used to reach the final \
answer.  For instance, consider the successive evaluations of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[Plus,{1,2,3,4,5}]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Trace[Apply[Plus,{1,2,3,4,5}]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["First ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" reduced the command to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1+2+3+4+5",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", and then it performed the addition operations and reached \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["15",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Now we use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to examine the successive evaluations when we execute \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[3]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Trace[fac[3]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["First ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" reduced the command to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3 fac[3 - 1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Then, separately, it evaluated \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3 - 1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  (The fact that a list appears inside the original list indicates the \
separate evaluation.)  In evaluating \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3 - 1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" reduced the expression to an addition, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-1 + 3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and added to yield \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Then ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" expanded \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", first by using the definition of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to produce \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2 fac[2 - 1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", then evaluating \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2 - 1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to produce \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".   At this stage ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" evaluated \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", using the specific definition, which results in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Then ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" substituted \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2 fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", yielding \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and substituted \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3 fac[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", yielding \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["6",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which became the final result.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If we are interested only in the successive invocations of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we can instruct \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to take out all steps that do not involve the expression \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  We do so by specifying \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" as the second parameter to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Trace[fac[3],fac]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Now pause for a moment and consider the result of the expression \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[2.5]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  What problem would arise?  What about \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[-3]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"?",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["These examples should indicate our function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" is an excellent candidate for data type checking.   A good solution to \
the problem of implementing a factorial function might be the following:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[fac];\nfac[1] = 1;\nfac[n_?((IntegerQ[#]&&(#>0))&)] := n fac[n-1];\n\
fac[n_] := Print[\"The argument \\\"\",n,\n\t\"\\\" is not a positive \
integer.\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["fac[-5]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["fac[5]"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Writing Readable Code: Comments, Usage Lines, and Style"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"As we create progressively more complicated code, keep in mind that, in many \
situations, the audience for your programming \"composition\" is not simply \
your machine; it may also include other programmers.  Programs are more \
easily modified and improved by others if the code is initially composed to \
be readable and modular.  Writing readable code can be accomplished by \
improving the code itself, and by using comments and usage lines."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData[
"Using Boolean Tests and Automatic Data Type Checking"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Recall that our last version of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" includes a rather complicated boolean test:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[fac];\nfac[1] = 1;\nfac[n_?((IntegerQ[#]&&(#>0))&)] := n fac[n-1];\n\
fac[n_] := Print[\"The argument \\\"\",n,\n\t\"\\\" is not a positive \
integer.\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["This code could be improved by highlighting the fact that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" needs to be an integer, away from the boolean test.  We can do so \
because ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " allows the use of both a data type check and a boolean test together.  \
The data type must be placed between the underscore and the following \
question mark, finally followed by the boolean test.  In the following \
example, using \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["_Integer?((#>0)&)",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" instead of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["_?(IntegerQ[#]&&(#>0))&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" makes our \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" function easier to read.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[fac];\nfac[1] = 1;\nfac[n_Integer?((#>0)&)] := \n\t\t\tn fac[n-1];\n\
fac[n_]:= \n\t\t\tPrint[\"The argument \\\"\",n,\n\t  \"\\\" is not a \
positive integer.\"]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["fac[-5]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["fac[5]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Comments, Usage Lines"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Comments"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Comments are a wonderful way of making a function readable.  A comment can \
be inserted anywhere within a cell or program and is formed by prepending the \
comment (any sequence of symbols) by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(*",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and appending it with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*)",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  In general, a judicious but not over-zealous use of comments can \
greatly improve a function.  In our factorial function, we might add comments \
thus:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\

(* factorial function, using recursion *)

(* fac[n]=n fac[n-1], with base case fac[1]=1 *)

Clear[fac]; 

fac[1] = 1;

fac[n_Integer?((#>0)&)] := n fac[n-1];
\t\t\t
fac[n_]:=  (* if n is not a positive integer *)
\tPrint[\"The argument \\\"\",n,
\t  \"\\\" is not a positive integer.\"]\
\>", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Usage Lines"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " feature which complements internal comments is the provision of a \"usage \
line\" for the function.  Recall that executing \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["?",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\" when \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\" is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " function yields a nice description of the function, while if we have \
previously defined \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[
  "\", then only our definition of the function is returned.  We can instruct \
",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " to return a description for our function by providing a string which \
describes the function and indicating that this string should be the usage \
line. Once we do so, the user can either ask for the usage line (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["?",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\") or insist on the full definition (using \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["??",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["function",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\").  Let's add a usage line for our factorial function:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\

(* factorial function, using recursion *)

(* fac[n]=n fac[n-1], with base case fac[1]=1 *)

Clear[fac]; 

fac::usage=
\t\"fac[n] computes n! for positive integers n\" ;

fac[1] = 1;

fac[n_Integer?Positive] := n fac[n-1];
\t\t\t
fac[n_]:=  (* if n is not a positive integer *)
\tPrint[\"The argument \\\"\",n,
\t  \"\\\" is not a positive integer.\"]\
\>", "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["(Note that we also replaced the anonymous function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["((#>0)&)",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Positive[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which performs the same operation.)",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["?fac"], "Input",
  AspectRatioFixed->True],

Cell[TextData["??fac"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Summary: Programming Style"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Good programming style incorporates the following guidelines, as \
well as many others.  To write code well, keep each of these in mind: 
\t(1) Use comments whenever the code might be unclear to the reader.  
\t(2) Write a usage line which explains your function.  
\t(3) Keep all intermediate results local, and control the side effects of \
your functions.  
\t(4) Don't allow a user to execute your function with bad data; check for \
the data before executing a particular definition.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"An Alternative to Boolean Parameter Tests:  Condition Notation"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Suppose we wish to define a function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["step[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" with one parameter, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which is equal to one for all ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x>2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" and equal to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-.5", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[" for all ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x<=2", "Input",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Plain"],
  StyleBox[
  ".  Using anonymous functions for boolean tests, we could write the \
following:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[step];\nstep[n_?((#>2)&)] := 1;\nstep[n_?((#<=2)&)] := -0.5;"], "Input",\

  AspectRatioFixed->True],

Cell[TextData["step[3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["step[-4]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " provides yet another way to express conditional evaluation, however; we \
can place what is called a condition immediately following an underscore or a \
data type check.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " will then use the specified definition only if the conditions on the \
parameters are valid.  The syntax of a condition requires the placement of a \
boolean expression (usually involving the parameter) after the symbols \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["/;",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\". (To place conditions after more than one argument, place the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["\\;",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" followed by the condition after the underscore or data type check and \
before the comma separating the parameters, as in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a_ \\; a>1, b_ \\; b<2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".)  Using conditions, we have the following alternate solution:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[step];\nstep[n_ /; n > 2] := 1;\nstep[n_ /; n <= 2] := -0.5;"], "Input",\

  AspectRatioFixed->True],

Cell[TextData["step[3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["step[-4]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "At this stage we may want to add a data type check.  We would like to \
constrain the function so that the value of the parameter must be a number, \
and we could use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["NumberQ[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" as part of our condition to test this fact.  However, this method will \
not incorporate all of the cases we would like:  we would like to be able to \
compute \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["step[Pi]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", even though \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["NumberQ[Pi]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is false.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["step[Pi]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["What we will do instead is find the value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["N[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" applied to the parameter, and then test whether or not this new value \
is a number.  If so, the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["step[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", function will return the appropriate value; if not, then it will \
return an error message.  We will also add a usage line and a comment.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[step]; (* step function *)\n\n(* step[x]=1 if x is greater than 2;\n   \
step[x]=-0.5 if x is less than or equal to 2;\n   step returns an error \
message if x does not\n   reduce to a number *)\n   \nstep::usage = \"step[n] \
returns 1 if n is greater than\n2 and -0.5 is n is less than 2\";\n\nstep[n_ \
/; NumberQ[N[n]]&&(N[n]>2)] := 1;\n\nstep[n_ /; NumberQ[N[n]]&&(N[n]<=2)] := \
-0.5;\n\nstep[n_] := Print[\"Argument \\\"\",n,\"\\\" is invalid.\"];\n\n"], 
  "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Structured Patterns"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Sometimes a programmer knows that the structure of a parameter extends \
beyond a simple data type or arithmetic value; indeed, the structure might \
always be a list with three elements. When this is the case, and the \
programmer wants to use each element separately, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides a way to name each of them as parameters, using \"structured \
patterns\".  A structured pattern is a parameter which is not a single data \
type but a specified, more complex structure, with parameters inside the \
pattern.  "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Suppose we wish to define a function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["switchTwo[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", which interchanges the first two elements of a list with three \
elements, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{x,y,z}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " (also called a triple).  Without a structured pattern, we might do the \
following:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[switchTwo]\nswitchTwo[triple_] :=\n   {triple[[2]], triple[[1]], \
triple[[3]]}"], "Input",
  AspectRatioFixed->True],

Cell[TextData["switchTwo[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"If we wanted insure that the value of the parameter was in fact a list with \
exactly three elements, we might go a bit further:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[switchTwo];\nswitchTwo[triple_?((ListQ[#]&&Length[#]==3)&)]:=\n\t\
{triple[[2]], triple[[1]], triple[[3]]};\nswitchTwo[triple_] := \n   \
Print[\"Needs a list of three elements.\"];"], "Input",
  AspectRatioFixed->True],

Cell[TextData["switchTwo[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "However, with a structured pattern, we can more compactly write the \
parameter as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{x_, y_, z_}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", with the three (sub-) parameters inside.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " will then check for an argument which matches this one exactly---a list \
with three elements (of any type).",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[switchTwo];\nswitchTwo[{x_,y_,z_}]:= {y,x,z};\nswitchTwo[n_] := \n   \
Print[\"Needs a list of three elements.\"];"], "Input",
  AspectRatioFixed->True],

Cell[TextData["switchTwo[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "This idea can be used to great advantage when we want to refer to specific \
parts of expected input parameters.  In a previous section we considered \
defining a function to determine the difference between the x and \
y-coordinates of a point ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{x,y}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ".  Without a structured pattern, we might use the following definition:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[xyDiff]\nxyDiff[pt_]:= Abs[ Last[pt] - First[pt] ]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"With a structured pattern, however, we have a much more readable solution."],
   "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[xyDiff]\nxyDiff[{x_,y_}]:= Abs[ y-x ]"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{520, 509},
WindowMargins->{{68, Automatic}, {30, Automatic}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000002n88o?mooh<<H`Ui0fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@00000000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 129, 3, 174, "Title",
  Evaluatable->False],
Cell[1863, 56, 122, 2, 136, "Title",
  Evaluatable->False],
Cell[1988, 60, 101, 2, 26, "SmallText",
  Evaluatable->False],
Cell[2092, 64, 913, 18, 146, "Text",
  Evaluatable->False],
Cell[3008, 84, 1141, 41, 138, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[4174, 129, 130, 3, 50, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[4329, 136, 98, 2, 46, "Subsection",
  Evaluatable->False],
Cell[4430, 140, 466, 8, 78, "Text",
  Evaluatable->False],
Cell[4899, 150, 1024, 35, 90, "Output",
  Evaluatable->False],
Cell[5926, 187, 358, 6, 78, "Text",
  Evaluatable->False],
Cell[6287, 195, 113, 2, 42, "Input"],
Cell[6403, 199, 633, 22, 47, "Text",
  Evaluatable->False],
Cell[7039, 223, 74, 1, 27, "Input"],
Cell[7116, 226, 2487, 73, 211, "Text",
  Evaluatable->False],
Cell[9606, 301, 117, 2, 42, "Input"],
Cell[9726, 305, 69, 1, 27, "Input"],
Cell[9798, 308, 74, 1, 27, "Input"],
Cell[9875, 311, 1067, 32, 99, "Text",
  Evaluatable->False],
Cell[10945, 345, 202, 4, 72, "Input"],
Cell[11150, 351, 100, 1, 57, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11287, 357, 127, 2, 46, "Subsection",
  Evaluatable->False],
Cell[11417, 361, 812, 17, 130, "Text",
  Evaluatable->False],
Cell[12232, 380, 244, 5, 87, "Input"],
Cell[12479, 387, 146, 3, 87, "Input"],
Cell[12628, 392, 1077, 33, 98, "Text",
  Evaluatable->False],
Cell[13708, 427, 855, 25, 80, "Text",
  Evaluatable->False],
Cell[14566, 454, 195, 4, 72, "Input"],
Cell[14764, 460, 485, 15, 47, "Text",
  Evaluatable->False],
Cell[15252, 477, 81, 1, 42, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[15382, 484, 124, 2, 30, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[15531, 490, 110, 2, 46, "Subsection",
  Evaluatable->False],
Cell[15644, 494, 2126, 66, 171, "Text",
  Evaluatable->False],
Cell[17773, 562, 284, 5, 62, "Text",
  Evaluatable->False],
Cell[18060, 569, 109, 2, 57, "Input"],
Cell[18172, 573, 76, 1, 27, "Input"],
Cell[18251, 576, 961, 30, 81, "Text",
  Evaluatable->False],
Cell[19215, 608, 86, 1, 57, "Input"],
Cell[19304, 611, 76, 1, 27, "Input"],
Cell[19383, 614, 545, 20, 32, "Text",
  Evaluatable->False],
Cell[19931, 636, 55, 1, 27, "Input"],
Cell[19989, 639, 468, 15, 47, "Text",
  Evaluatable->False],
Cell[20460, 656, 189, 6, 72, "Input"],
Cell[20652, 664, 67, 1, 27, "Input"],
Cell[20722, 667, 63, 1, 27, "Input"],
Cell[20788, 670, 804, 25, 80, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[21629, 700, 146, 4, 46, "Subsection",
  Evaluatable->False],
Cell[21778, 706, 324, 6, 62, "Text",
  Evaluatable->False],
Cell[22105, 714, 100, 1, 57, "Input"],
Cell[22208, 717, 59, 1, 27, "Input"],
Cell[22270, 720, 1610, 58, 83, "Text",
  Evaluatable->False],
Cell[23883, 780, 986, 32, 81, "Text",
  Evaluatable->False],
Cell[24872, 814, 83, 1, 27, "Input"],
Cell[24958, 817, 1169, 42, 66, "Text",
  Evaluatable->False],
Cell[26130, 861, 66, 1, 27, "Input"],
Cell[26199, 864, 4965, 183, 171, "Text",
  Evaluatable->False],
Cell[31167, 1049, 1194, 41, 65, "Text",
  Evaluatable->False],
Cell[32364, 1092, 70, 1, 27, "Input"],
Cell[32437, 1095, 601, 21, 48, "Text",
  Evaluatable->False],
Cell[33041, 1118, 507, 15, 63, "Text",
  Evaluatable->False],
Cell[33551, 1135, 213, 4, 87, "Input"],
Cell[33767, 1141, 60, 1, 27, "Input"],
Cell[33830, 1144, 59, 1, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[33938, 1151, 133, 3, 48, "Section",
  Evaluatable->False],
Cell[34074, 1156, 522, 8, 94, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[34621, 1168, 133, 3, 46, "Subsection",
  Evaluatable->False],
Cell[34757, 1173, 390, 13, 31, "Text",
  Evaluatable->False],
Cell[35150, 1188, 213, 4, 87, "Input"],
Cell[35366, 1194, 1451, 47, 115, "Text",
  Evaluatable->False],
Cell[36820, 1243, 222, 4, 117, "Input"],
Cell[37045, 1249, 60, 1, 27, "Input"],
Cell[37108, 1252, 59, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[37204, 1258, 101, 2, 46, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[37330, 1264, 91, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[37424, 1268, 858, 26, 95, "Text",
  Evaluatable->False],
Cell[38285, 1296, 339, 16, 222, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[38661, 1317, 94, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[38758, 1321, 2574, 88, 152, "Text",
  Evaluatable->False],
Cell[41335, 1411, 403, 19, 267, "Input"],
Cell[41741, 1432, 774, 27, 49, "Text",
  Evaluatable->False],
Cell[42518, 1461, 57, 1, 27, "Input"],
Cell[42578, 1464, 58, 1, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[42685, 1471, 106, 2, 46, "Subsection",
  Evaluatable->False],
Cell[42794, 1475, 560, 11, 126, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[43403, 1492, 140, 3, 48, "Section",
  Evaluatable->False],
Cell[43546, 1497, 1198, 43, 64, "Text",
  Evaluatable->False],
Cell[44747, 1542, 122, 3, 57, "Input"],
Cell[44872, 1547, 60, 1, 27, "Input"],
Cell[44935, 1550, 61, 1, 27, "Input"],
Cell[44999, 1553, 1605, 48, 149, "Text",
  Evaluatable->False],
Cell[46607, 1603, 122, 3, 57, "Input"],
Cell[46732, 1608, 60, 1, 27, "Input"],
Cell[46795, 1611, 61, 1, 27, "Input"],
Cell[46859, 1614, 1031, 33, 96, "Text",
  Evaluatable->False],
Cell[47893, 1649, 61, 1, 27, "Input"],
Cell[47957, 1652, 773, 24, 80, "Text",
  Evaluatable->False],
Cell[48733, 1678, 521, 8, 267, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[49291, 1691, 96, 2, 30, "Section",
  Evaluatable->False],
Cell[49390, 1695, 644, 13, 112, "Text",
  Evaluatable->False],
Cell[50037, 1710, 694, 24, 64, "Text",
  Evaluatable->False],
Cell[50734, 1736, 139, 3, 57, "Input"],
Cell[50876, 1741, 71, 1, 27, "Input"],
Cell[50950, 1744, 203, 4, 46, "Text",
  Evaluatable->False],
Cell[51156, 1750, 241, 4, 87, "Input"],
Cell[51400, 1756, 71, 1, 27, "Input"],
Cell[51474, 1759, 735, 24, 65, "Text",
  Evaluatable->False],
Cell[52212, 1785, 174, 3, 72, "Input"],
Cell[52389, 1790, 71, 1, 27, "Input"],
Cell[52463, 1793, 632, 18, 79, "Text",
  Evaluatable->False],
Cell[53098, 1813, 110, 2, 42, "Input"],
Cell[53211, 1817, 152, 4, 30, "Text",
  Evaluatable->False],
Cell[53366, 1823, 96, 1, 42, "Input"]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

