(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     34620,       1096]*)
(*NotebookOutlinePosition[     35697,       1131]*)
(*  CellTagsIndexPosition[     35653,       1127]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData["16. Recursion I:  Passing the Buck"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision:  February 8 1999", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"In this section we consider in more depth the programming style mentioned \
several times previously, recursion.  We define base cases and discuss how a \
recursive definition must implicitly assume that the function works correctly \
for smaller, or simpler, values of the parameter.  We conclude with good \
examples of recursion, as well as examples of programming styles which do not \
incorporate recursion."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["recursive definitions",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\n     First[]\n     Rest[]\n     Last[]\n     While[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Recursion"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["The Religion of Recursion:  Passing the Buck"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["recursive",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " function is a function which accomplishes a programming task by \
repeatedly invoking, or calling, itself.  The example of recursion which we \
have encountered several times before is the factorial function: ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[fac]\nfac[1]=1;\nfac[n_Integer /; Positive[n] ] := n fac[n-1]\n\
fac[7]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["In the definition of the function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" we find a function call to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which distinguishes this function as recursive.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Recursive functions differ from other functions in the way they break a \
programming task into smaller steps.  The functions which we have previously \
encountered used commands such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Map[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to repeat one operation over and over.  Instead of using such \
pre-defined commands to implement repetition, however, recursive functions \
accomplish a sequence of smaller steps by either (a) performing one small \
operation and then invoking themselves again, or (b) invoking themselves \
again and then performing one small operation.  Since one operation is \
accomplished with every invocation of the function, and since the function \
invokes itself repeatedly, the many small operations which make up the \
programming task are eventually accomplished.  In the factorial function \
above, the recursive call is of the second type, where the function invokes \
itself and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["then",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" performs the small operation.  What happens is that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" invokes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" again with a smaller parameter, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n-1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", and once the result is completed, the small operation is performed:  \
the result of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is multiplied by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Let us step through the action of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" so that we see clearly the successive, nested function calls.  We \
consider the action anthropomorphically.  When we enter \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[7]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" says, \"I don't want to do all this work.  I'll pass most of the burden \
on to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[6]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  I'll just multiply whatever ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[6]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" gives me by 7.\"  Then execution for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[7]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is suspended, while \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[7]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" waits for the result from \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[6]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  When \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" is called with a parameter of \"6\", it says, \"I don't want to do all \
this work, let me get that lackey ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[5]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " to do most of the work, then I'll just multiply by 6.\"  Then execution \
for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[6]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is suspended, waiting on the result of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[5]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  In turn, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[5]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" gets the underling \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[4]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to do most of the work, and so on.   All of this buck passing finally \
comes to a end with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", who has no assistants to turn to.  But \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" has a special definition:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]=1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Hence \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" returns a 1, and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" resumes execution, taking this result, 1, and multiplying by 2.  Then \
\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[3]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" resumes execution, taking the result of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", namely 2, and multiplying it by 3, and so forth.  Finally \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[7]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" takes the result of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[6]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", namely 720, and multiplies by 7, yielding the final answer, 5040. ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData[
"Base Cases; Simplifying until the Base Case Holds"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "From studying the function's repeatedly invoking itself, it should be \
clear that having a special case, where \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]=1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", was very important; otherwise, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" would have passed the buck an infinite number of times!  This sort of \
special case for a recursive function, in which a fixed return value is \
specified for a particular value of the parameter, is called a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["base case",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  Our function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", then, has a base case when the value of the parameter is equal to 1, \
and the result of the base case is to return the integer 1.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Typically, recursive functions operate by simplifying the parameter(s) and \
then invoking themselves again with a simpler parameter value, where the \
meaning of \"simpler\" depends on the parameter type.  For instance, if the \
parameter values are positive integers, as in the case above, a simpler \
parameter value would be a smaller, positive integer.  On the other hand, if \
the parameter values are lists, then a simpler parameter value might be a \
list with fewer elements.  The value of the function for the base case is \
thus defined to be the result when the parameter is the simplest possible.  \
Again, since the recursive function repeatedly simplifies the parameter, the \
process of buck-passing is destined to stop, since the simplest possible \
case, the base case, will ultimately be reached.  In the factorial function, \
the simplest possible case is when the parameter value is a 1.  "], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Writing Recursive Functions"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "In practice, when writing a recursive function, it is usually best to work \
from the bottom up, which here means that one should define the base case ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["first",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ".  In other words, decide what values the parameter should allow, the way \
in which the parameters might become simpler, and what the simplest possible \
case would be.  Then write the definition of the recursive function, which \
must, somewhere in its definition, invoke itself with a simpler version of \
the parameter.  When defining the recursive function, one must have faith \
that the function will perform correctly for simpler parameters; the job of \
the function is only to perform a small operation and invoke the function \
again with a simpler parameter.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Let us consider this process, applied to the factorial function above.  We \
would first decide that the factorial function takes positive integer \
parameters, the simplest of which would be the smallest positive integer, \
namely 1.  Next, we would define the base case:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[1]=1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Finally, we would consider what small operation \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" should perform, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["assuming that \"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["fac[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\" applied to a smaller value returns the correct value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  If \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" has the correct value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", then we notice that we need only multiply \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and our definition of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" will be correct.  We thus define the factorial function to invoke \
itself with a parameter value which is simpler, namely one less than the \
original value, and then to take the result of that function call and \
multiply its value by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Hence the definition is \n\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fac[n] := n fac[n-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Examples:  Zero Tolerance"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["No preceding zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Suppose we want to eliminate all zeros at the beginning of a list.  We seek \
a function, therefore, which takes a list and returns a list with the zeros \
at the beginning removed.  The parameter values of the function are then \
lists, and a good place to begin for a notion of \"simpler\" parameters is to \
examine the length of the list.  The base case, the simplest case for which \
our function must perform the operation, is an empty list (a list with no \
elements), and in that case there is nothing to do but return the list, since \
it had no zeros to eliminate.  Let's go ahead and define our function for \
this base case."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Clear[elimZero]\nelimZero[{}] := {}"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Now we assume that our function will work on any \"small\" list; we need \
only decide what it should do with a \"larger\" list:  what piece of the \
programming task it should perform, and how it should invoke the function \
again with a simpler list.  We may reason as follows:  if a list has a \
beginning zero, we should drop it and invoke our function again with this new \
list; otherwise we will just return the list itself.  Note that we are \
applying the first type of recursion stated above, in which the function \
performs an operation first and then invokes the function again.  We \
implement our procedure as follows:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"elimZero[L_List] := \n\t\t\tIf[First[L]==0,elimZero[Rest[L]],L]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Thus our function takes a list and examines the first element.  If the \
first element is a zero, then it \"drops\" it---by taking only the remainder \
of the list---and applies the function again to this resulting list, \
returning the result of this invocation.  Otherwise, the function simply \
returns the entire list.  Note that we are making explicit use of the \
functions \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["First[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which returns the first element of a list, and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Rest[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which returns the list without its first element.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["elimZero[{0,0,0,4,5}]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["No ultimate zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["How would we rewrite \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["elimZero[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to perform similarly, removing zeros not from the beginning but from \
the end of the list?  We would simply have the function consider the last \
element of the list instead of the first, calling the function again if the \
last element was a zero.  We can use the function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Last[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to extract the last element of the list, but we will need to use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Drop[  ,-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to extract all of the elements of the list save the last.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[elimZeroAtEnd]\nelimZeroAtEnd[{}] := {}\nelimZeroAtEnd[L_List] :=\n\t\t\
\tIf[Last[L]==0,elimZeroAtEnd[Drop[L,-1]],L]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["elimZeroAtEnd[{4,5,0,0,0}]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["No zeros at all"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Suppose now that we want to rid a list of all zeros.  How could we do this \
recursively?  (We could surely use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Select[list,(#!=0)&]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", but that's not recursive.)  Here we must step back.  The base case is \
the same---a parameter value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and a return value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"---but we must consider what small operation our recursive function must \
accomplish, and how we will simplify the value of the parameter.  One way to \
start is to surmise that the function should consider one element of the list \
at a time.  If zero, our function should not return it; if not, our function \
should return it.  But this line of thought is not yet recursive, since we do \
not want our function to consider each element of the list, one at a time; \
instead, we want our function to perform one operation, say by deciding what \
to do with one element of the list, and then to invoke itself again, say with \
the rest of the list.  We think thus in terms of entire lists where we \
envision our function's examining only the first element.  This better line \
of thought proceeds as follows:  if the first element is a zero, we  take the \
element off and apply our function again to the new list; if the first \
element is not a zero, we still apply our function to the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["rest",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" of the list but ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["then",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " take the resulting list and put the first element back on.  Note that we \
are certainly allowed to manipulate the result of a call to a recursive \
function!  Let's take a look.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[killZero]\nkillZero[{}] := {}\nkillZero[L_List] := \n   \
If[First[L]==0,killZero[Rest[L]],\n   Prepend[killZero[Rest[L]],First[L]]]"], 
  "Input",
  AspectRatioFixed->True],

Cell[TextData[
"If the first element of the list is a zero, we return the result of applying \
our function again to the remainder of the list.  If the first element of the \
list is not zero, we must alter the result of applying our function to the \
remainder of the list, by placing the first element back on."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["killZero[{3,4,0,5,6,0,7,7,0,7,0}]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Counting the zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Now let's consider how we might recursively count all of the zeros in a \
list.  (Again, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Count[list,0]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" simply answers the question, but, who knows, maybe \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Count[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" has a recursive definition!)  Thinking recursively, we see that we want \
our function to consider one element of a list and then pass the rest of the \
list on to the function again.  What changes now is that instead of returning \
lists, we will be returning numbers, namely, the number of zeros found in the \
list.  We can define the function recursively, then, by checking to see if \
the first element of the list is a zero, and, if so, adding 1 to the result \
of applying the function to the rest of the list, and if not, adding 0 to the \
result of applying the function to the rest of the list.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[countZeros]\ncountZeros[{}] := 0\ncountZeros[L_List] := \n  \
If[First[L]==0,1+countZeros[Rest[L]],\n  \t\t\tcountZeros[Rest[L]]\n  ]"], 
  "Input",
  AspectRatioFixed->True],

Cell[TextData["countZeros[{4,5,0,0,3,4}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Notice that the return value of the definition of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["countZeros[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is an Integer, and that the base case returns a 0.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["What Recursion is Not"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "We have considered several examples of recursion, and it will be useful to \
know what sorts of code will not qualify, although they do return the correct \
answers.  First, use of a repetition function (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Map[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MapThread[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Nest[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["NestList[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Fold[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["FoldList[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Apply[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", and so on, being our \"adverbs\") will generally disqualify a method \
as purely recursive, for the repetition is accomplished externally from the \
nested function calls.  Also, repetition accomplished with a loop structure, \
such as a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["While[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", or some other repetition command, is explicitly forbidden.  Here are \
some examples of non-recursive solutions to the problems above.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["No preceding zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[elimZero]\nelimZero[l_List] := Module[{testAndAppend},\n\t\t\t\
testAndAppend[l1_List, l2_] := \t\t\t\n\t\t\t   If[l2==0,l1,Append[l1,l2]];\n\
\t\t\tFold[testAndAppend,{},l]\n\t\t\t]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["elimZero[{0,0,0,4,5}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here we're explicitly using \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Fold[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to accomplish the repetition.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["No ultimate zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[elimZeroAtEnd]\nelimZeroAtEnd[l_List] := Module[{c,d},\n\t\t\t\
c=Length[l]; d=0;\n\t\t\tWhile[c>=1,\n\t\t\t\t\t If[l[[c]]==0,d++]\n\t\t\t\t\t\
 c--\n\t\t\t];\n\t\t\tDrop[l,-d]\n\t\t\t]\n\t\t\t\t\t\t"], "Input",
  AspectRatioFixed->True],

Cell[TextData["elimZeroAtEnd[{4,5,0,0,0}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here we are using a special repetition construct, the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["While",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" construct, to count the number of zeros at the end, and then dropping \
them all at once.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["No zeros at all"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[killZero]\nkillZero[l_List] :=\n   l[[\n      Flatten[\n         \
Position[l,_?((#!=0)&)]\n      ]\n    ]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["killZero[{3,4,0,5,6,0,7,7,0,7,0}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Here we are running through the list ahead of time to find the nonzero \
entries and then choosing them out of the list."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Counting the zeros"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[countZeros]\ncountZeros[l_List] := \n   Apply[Plus,\n   \t\t\tTable[ \
If[l[[i]]==0,1,0] ,{i,1,Length[l]} ]\n   ]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["countZeros[{4,5,0,0,3,4}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here we are making \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" do our work for us.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{520, 509},
WindowMargins->{{92, Automatic}, {Automatic, 16}},
PrintingCopies->1,
PrintingPageRange->{1, 2},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003809T?o>old</PW30fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@CYh^0000000000060H01T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 109, 2, 136, "Title",
  Evaluatable->False],
Cell[1843, 55, 100, 2, 26, "SmallText",
  Evaluatable->False],
Cell[1946, 59, 487, 8, 94, "Text",
  Evaluatable->False],
Cell[2436, 69, 678, 23, 96, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3139, 96, 86, 2, 50, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3250, 102, 124, 2, 46, "Subsection",
  Evaluatable->False],
Cell[3377, 106, 527, 16, 64, "Text",
  Evaluatable->False],
Cell[3907, 124, 131, 3, 50, "Input"],
Cell[4041, 129, 599, 20, 25, "Text",
  Evaluatable->False],
Cell[4643, 151, 2595, 77, 142, "Text",
  Evaluatable->False],
Cell[7241, 230, 5210, 181, 155, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[12476, 415, 133, 3, 33, "Subsubsection",
  Evaluatable->False],
Cell[12612, 420, 1343, 41, 77, "Text",
  Evaluatable->False],
Cell[13958, 463, 985, 14, 116, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[14980, 482, 110, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[15093, 486, 1037, 23, 90, "Text",
  Evaluatable->False],
Cell[16133, 511, 2878, 93, 116, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[19060, 610, 105, 2, 42, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[19190, 616, 101, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[19294, 620, 711, 11, 77, "Text",
  Evaluatable->False],
Cell[20008, 633, 88, 1, 30, "Input"],
Cell[20099, 636, 708, 11, 77, "Text",
  Evaluatable->False],
Cell[20810, 649, 117, 2, 30, "Input"],
Cell[20930, 653, 972, 26, 64, "Text",
  Evaluatable->False],
Cell[21905, 681, 74, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[22016, 687, 100, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[22119, 691, 1097, 32, 64, "Text",
  Evaluatable->False],
Cell[23219, 725, 177, 3, 50, "Input"],
Cell[23399, 730, 79, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23515, 736, 98, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[23616, 740, 2438, 61, 168, "Text",
  Evaluatable->False],
Cell[26057, 803, 195, 4, 60, "Input"],
Cell[26255, 809, 371, 6, 51, "Text",
  Evaluatable->False],
Cell[26629, 817, 86, 1, 22, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[26752, 823, 101, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[26856, 827, 1248, 30, 90, "Text",
  Evaluatable->False],
Cell[28107, 859, 196, 4, 70, "Input"],
Cell[28306, 865, 78, 1, 22, "Input"],
Cell[28387, 868, 422, 13, 25, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[28858, 887, 101, 2, 42, "Subsection",
  Evaluatable->False],
Cell[28962, 891, 2492, 85, 90, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[31479, 980, 101, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[31583, 984, 242, 4, 70, "Input"],
Cell[31828, 990, 74, 1, 22, "Input"],
Cell[31905, 993, 373, 13, 25, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[32315, 1011, 100, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[32418, 1015, 259, 4, 110, "Input"],
Cell[32680, 1021, 79, 1, 22, "Input"],
Cell[32762, 1024, 461, 15, 38, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[33260, 1044, 98, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[33361, 1048, 167, 3, 80, "Input"],
Cell[33531, 1053, 86, 1, 22, "Input"],
Cell[33620, 1056, 195, 4, 25, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[33852, 1065, 101, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[33956, 1069, 173, 3, 60, "Input"],
Cell[34132, 1074, 78, 1, 22, "Input"],
Cell[34213, 1077, 355, 13, 25, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

