(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     30817,       1043]*)
(*NotebookOutlinePosition[     31852,       1076]*)
(*  CellTagsIndexPosition[     31808,       1072]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData["20. Arrays and Matrices"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision:  March 19 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"In this lab we introduce a useful new data type, called an array.  We \
consider arrays, methods for their definition, initialization, and \
manipulation, and pay particular attention to using iterative structures to \
modify them."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["array\n     vector",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\n     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["size\n     indices",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[
  "\n     Dimensions[]\n     ColumnForm[]\n     MatrixForm[]\n     \
IdentityMatrix[]\n     DiagonalMatrix[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Arrays"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["An ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["array",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["  is a general data type.  A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["one-dimensional array",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["  is simply a list of elements, such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{2,3,4}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"; such an array is sometimes termed a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["vector",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["two-dimensional array",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  "  is a list of lists of elements, where the lists inside all have the same \
length, such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{{1,2},{4,5},{7,8}}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"; such an array is sometimes termed a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrix",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".   A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["three-dimensional array ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " is then a list of lists of lists of elements, where the innermost lists \
all have the same length and the lists containing those lists also all have \
the same length.  For an example of a three-dimensional array, consider the \
expression \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{{{1,2,3},{2,3,4}},{{3,4,5},{5,6,7}},{{7,8,9},{8,9,10}}}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Note that the innermost lists all have length 3, and the lists \
containing these innermost lists all have length 2 (containing 2 lists of \
length 3).  Arrays can be defined for any positive integral dimension by \
continuing in the same format.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Sizes"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["As you may surmise, arrays have a definite",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" size",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ".  For one-dimensional arrays, the size is simply the number of elements.  \
For two-dimensional arrays, however, the size must contain two numbers: the \
number of inner lists, followed by the number of elements in each list.  For \
three-dimensional arrays, the size is composed of three numbers: the number \
of outermost lists, the number of lists inside a given outermost list, and \
finally the number of elements in each of the innermost lists.  In the \
examples above, the one-dimensional array has size 3; the two-dimensional \
array has size \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3,2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" (or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3x2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["); and the three-dimensional array size \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3,2,3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" (or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3x2x3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[").  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " can determine these sizes, using a command we have considered some time \
ago, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Dimensions[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Let's try it:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"oneDArray = {2,3,4};\ntwoDArray = {{1,2},{4,5},{7,8}};\nthreeDArray = \
{{{1,2,3},{2,3,4}},{{3,4,5},{5,6,7}},\n{{7,8,9},{8,9,10}}};\n\n\
Dimensions[oneDArray]\nDimensions[twoDArray]\nDimensions[threeDArray]"], 
  "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " also has some functions which display arrays in rectangular ways.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["ColumnForm[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" displays a vector as a column instead of a row, and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MatrixForm[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" displays a two-dimensional array in rectangular form, where the inner \
lists are written out as rows.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["ColumnForm[oneDArray]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["MatrixForm[twoDArray]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"(For obvious reasons, there is no simple way to display an array of \
dimensionality greater than two.)"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Individual Elements"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We can refer to individual elements of a list with",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" array indices",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ".  These indices tell us how to find the element by indicating, working \
from the outside in, which particular list, which element of if that list, \
and so on, contains the element we are interested in. In a one-dimensional \
array, we need only mention one number, the number of elements from the \
beginning of the list one must count to find the element.  In a \
two-dimensional array, we require two numbers:  the first index will tell us \
which list to use (which \"row\"), the second which element of that list \
(which \"column\").  For instance, in the two-dimensional array \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{{1,2,3},{4,5,6}}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" the element with value 3 has indices \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1,3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Similarly, in a three-dimensional array, the three indices pinpoint, \
respectively, which outer list, which list of that list, and which element of \
that inner list one must consider to reach the element in question.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "To determine what value resides in an array at a particular set of array \
indices, we can use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Part[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" or its functional equivalent, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["[[]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\":",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["oneDArray[[2]]\nPart[oneDArray,2]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["twoDArray[[3,2]]\nPart[twoDArray,3,2]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"Note that if we specify too few indices, you will get a list or list of \
lists---all of the elements of the array whose first so many indices match \
our specified ones."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Part[threeDArray,2,2]"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Arrays: Declaration and Initialization"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "To declare that a name refers to an array, we must decide from the outset \
what the size of the array will be and what values we will initially assign \
to its elements.  Initializing an array to be an array of zeros is simplest, \
and we will begin thus.  To initialize a one-dimensional array of zeros, we \
use a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" command.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["myArray = Table[0,{10}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"For two-dimensional and three-dimensional arrays, we use multiple \
iterators."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["my2 = Table[0,{10},{3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["my3 = Table[0,{3},{2},{3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "It may happen that we know some nonzero values to which we would like the \
array initialized.  We can use \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to initialize the values of the various entries to the results of some \
function or expression, which may depend on the indices of the entries.  We \
might want to initialize each entry of a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3x5",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" array to its own row number; in that case, we would execute ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["my4 = Table[i,{i,3},{j,5}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Similarly, to initialize a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["4x6",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" array with value \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i+2^j",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" at indices \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i,j",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", we would execute",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[f]\nf[i_,j_]:=i+2^j;\nmy5 = Table[f[i,j],{i,4},{j,6}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["or"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["my5 = Table[i+2^j,{i,4},{j,6}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["It is also possible to define special sorts of arrays in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  An ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["identity matrix ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" is a two-dimensional array of size \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n,n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" which contains all zeros, except that the elements with two identical \
indices have value 1.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" can create such a matrix using the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["IdentityMatrix[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" command.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["IdentityMatrix[5]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["More generally, to specify a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["diagonal matrix",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", which is a square two-dimensional array which contains all zeros,  \
except that the elements with two identical indices may have nonzero values.  \
",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" can create such a matrix using the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["DiagonalMatrix[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" command.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["DiagonalMatrix[{1,2,3}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Can you tell how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " decides what size the diagonal matrix should be?"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Arrays: Modification"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"We can modify the value of any array element simply by executing an \
assignment statement, referring to an element via its indices."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["oneDArray"], "Input",
  AspectRatioFixed->True],

Cell[TextData["oneDArray[[2]]=500"], "Input",
  AspectRatioFixed->True],

Cell[TextData["oneDArray"], "Input",
  AspectRatioFixed->True],

Cell[TextData[
"In many cases, though, instead of modifying a particular array's elements, \
we may wish to have a copy of the old array made, and then modify the \
elements in the new array.  (One particular case where we will need to do so \
occurs when an array is an actual parameter in a function call; since we \
cannot modify the value of a formal parameter inside a function, we must copy \
the formal parameter to a local variable in order to permit modifications.)  \
We can do so by using an assignment statement to complete the first copy of \
matrices."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["twoDArray"], "Input",
  AspectRatioFixed->True],

Cell[TextData["twoD2 = twoDArray;\ntwoD2[[3,2]]=0;\ntwoD2"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Arrays are frequently manipulated using iterative structures such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["For[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Here are some examples.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Example 1: Adding to Every Element"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Let's add the integer ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " to each element of a vector, by individually assigning each entry.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"addOne[v_List] := Module[{newv},\n   newv = v;\n   Do[\n      newv[[i]] = \
v[[i]] + 1\n   ,{i,1,Length[v]}];\n   newv\n]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["addOne[{2,7,3,4,9}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "(Of course, this process is much simpler using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s list-based operations.)"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["{2,7,3,4,9}+1"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that we had to copy vector \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["v",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["newv",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in order to modify it; formal parameters such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["v",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" cannot be modified inside a function.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Example 2: Cumulative Sums"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A more interesting example is to take a vector and find the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["cumulative sum vector",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", made up of sums of consecutive elements beginning with the first \
element, so that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["cumSum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{2,7,3,4,9}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" would return \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["{2,9,12,16,25}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Using Do and Assignment of Elements"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "cumSum[v_List] := Module[{sumv},\n   sumv = v; ",
  StyleBox["(* copy the vector *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n   Do[ ",
  StyleBox["(* successively add vector elements *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n      sumv[[i]] = sumv[[i-1]] + v[[i]]\n   ,{i,2,Length[v],1}];\n   sumv \
",
  StyleBox["(* return answer *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n]  "
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["cumSum[{2,7,3,4,9}]"], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Using Do and Append"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Clear[cumSum];\ncumSum[v_List] := Module[{sumv},\n   sumv = {v[[1]]}; ",
  StyleBox["(* get the first entry *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n   Do[\n      sumv = Append[sumv, sumv[[i-1]] + v[[i]] ]\n   \
,{i,2,Length[v],1}];\n   sumv\n]  "
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["cumSum[{2,7,3,4,9}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["One advantage of the first version of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["cumSum[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is that it does not use any list functions (such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Append[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\") except \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Length[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Many languages support arrays but no list functions; thus, the first \
version is programmable in almost any language.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Example 3: Exchanging Two Rows"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Suppose we have two matrices (two-dimensional arrays) which have the same \
number of columns, and we wish to exchange row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" from the first matrix with row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["j",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" of the second.  Our strategy will be to copy row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" from the first matrix to another temporary array, to copy row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["j",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" from the second matrix to row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" of the first, and finally to copy our temporary array to row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" of the first.  We will assume we are given two matrices as arguments \
together with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["j",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"; we will return the two new matrices in a list.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Clear[rowSwitch]\nrowSwitch[m1_,m2_,i_,j_] := Module[\n\t\t\
{newm1=m1,newm2=m2,temp,cols,k}, ",
  StyleBox["(* init newm1,2 *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\tIf[Length[Dimensions[m1]]!=2||\n\t\t   Length[Dimensions[m2]]!=2,\n\t\
\t   Print[\"Arrays not 2-dimensional\"],\n\t\t   \
If[Dimensions[m1][[2]]!=Dimensions[m2][[2]],\n\t\t      Print[\"Number of \
columns different\"],\n\t\t      cols = Dimensions[m1][[2]]; ",
  StyleBox["(* find # cols *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t      temp = Table[0,{cols}]; ",
  StyleBox["(* initialize temp *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t      For[k=1, k<=cols, k++, ",
  StyleBox["(* k from 1 to cols *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t         temp[[k]]=newm1[[i,k]]\n\t\t         ]; ",
  StyleBox["(* now temp has old row i *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t      For[k=1, k<=cols, k++,\n\t\t      \t  newm1[[i,k]] = \
newm2[[j,k]]\n\t\t      \t  ]; ",
  StyleBox["(* now row i is old row j *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t      For[k=1, k<=cols, k++,\n\t\t         newm2[[j,k]] = temp[[k]]\n\
\t\t         ]; ",
  StyleBox["(* now row j is temp *)",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  "\n\t\t      {newm1,newm2}\n\t\t   ]\n\t\t]\n\t\t];"
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["rowSwitch[twoD2,twoDArray,2,3]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Study the code carefully.  Note again that local variables were used \
instead of formal parameters so that modifications could be made.  Some will \
notice that a simpler solution exists, which we give below, which uses the \
fact that we can reference an entire row of a two-dimensional matrix by \
specifying only the row number between double brackets.  For instance, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["newm1[[i]]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" refers to the entire list of row \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in array \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["newm1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Clear[rowSwitch2]\nrowSwitch2[m1_,m2_,i_,j_] := Module[\n\t\t\
{newm1=m1,newm2=m2,temp,cols,k}, (* init newm1,2 *)\n\t\t\
If[Length[Dimensions[m1]]!=2||\n\t\t   Length[Dimensions[m2]]!=2,\n\t\t   \
Print[\"Arrays not 2-dimensional\"],\n\t\t   \
If[Dimensions[m1][[2]]!=Dimensions[m2][[2]],\n\t\t      Print[\"Number of \
columns different\"],\n\t\t      temp = newm1[[i]];\n\t\t      newm1[[i]] = \
newm2[[j]];\n\t\t      newm2[[j]] = temp;\n\t\t      {newm1,newm2}\n\t\t   ]\n\
\t\t]\n\t\t];"], "Input",
  AspectRatioFixed->True],

Cell[TextData["rowSwitch2[twoD2,twoDArray,2,3]"], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Macintosh",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{625, 489},
WindowMargins->{{Automatic, 35}, {Automatic, 16}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000002n88o?mooh<<H`Ui0fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@00000000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 98, 2, 98, "Title",
  Evaluatable->False],
Cell[1840, 55, 98, 2, 26, "SmallText",
  Evaluatable->False],
Cell[1941, 59, 306, 6, 46, "Text",
  Evaluatable->False],
Cell[2250, 67, 969, 34, 152, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3244, 105, 83, 2, 50, "Section",
  Evaluatable->False],
Cell[3330, 109, 2528, 79, 150, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[5883, 192, 85, 2, 46, "Subsection",
  Evaluatable->False],
Cell[5971, 196, 1636, 49, 112, "Text",
  Evaluatable->False],
Cell[7610, 247, 524, 19, 48, "Text",
  Evaluatable->False],
Cell[8137, 268, 263, 5, 132, "Input"],
Cell[8403, 275, 834, 27, 64, "Text",
  Evaluatable->False],
Cell[9240, 304, 74, 1, 27, "Input"],
Cell[9317, 307, 74, 1, 27, "Input"],
Cell[9394, 310, 178, 4, 30, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[9609, 319, 99, 2, 46, "Subsection",
  Evaluatable->False],
Cell[9711, 323, 1572, 38, 144, "Text",
  Evaluatable->False],
Cell[11286, 363, 616, 22, 46, "Text",
  Evaluatable->False],
Cell[11905, 387, 86, 1, 42, "Input"],
Cell[11994, 390, 90, 1, 42, "Input"],
Cell[12087, 393, 245, 5, 46, "Text",
  Evaluatable->False],
Cell[12335, 400, 74, 1, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12458, 407, 115, 2, 30, "Section",
  Evaluatable->False],
Cell[12576, 411, 645, 18, 62, "Text",
  Evaluatable->False],
Cell[13224, 431, 76, 1, 27, "Input"],
Cell[13303, 434, 153, 4, 30, "Text",
  Evaluatable->False],
Cell[13459, 440, 76, 1, 27, "Input"],
Cell[13538, 443, 79, 1, 27, "Input"],
Cell[13620, 446, 849, 25, 78, "Text",
  Evaluatable->False],
Cell[14472, 473, 79, 1, 27, "Input"],
Cell[14554, 476, 725, 27, 30, "Text",
  Evaluatable->False],
Cell[15282, 505, 113, 2, 57, "Input"],
Cell[15398, 509, 76, 2, 30, "Text",
  Evaluatable->False],
Cell[15477, 513, 83, 1, 27, "Input"],
Cell[15563, 516, 1259, 43, 66, "Text",
  Evaluatable->False],
Cell[16825, 561, 70, 1, 27, "Input"],
Cell[16898, 564, 901, 30, 66, "Text",
  Evaluatable->False],
Cell[17802, 596, 76, 1, 27, "Input"],
Cell[17881, 599, 204, 7, 32, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[18122, 611, 97, 2, 30, "Section",
  Evaluatable->False],
Cell[18222, 615, 207, 4, 46, "Text",
  Evaluatable->False],
Cell[18432, 621, 62, 1, 27, "Input"],
Cell[18497, 624, 71, 1, 27, "Input"],
Cell[18571, 627, 62, 1, 27, "Input"],
Cell[18636, 630, 624, 10, 94, "Text",
  Evaluatable->False],
Cell[19263, 642, 62, 1, 27, "Input"],
Cell[19328, 645, 95, 1, 57, "Input"],
Cell[19426, 648, 765, 28, 46, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[20216, 680, 114, 2, 46, "Subsection",
  Evaluatable->False],
Cell[20333, 684, 398, 14, 30, "Text",
  Evaluatable->False],
Cell[20734, 700, 175, 3, 117, "Input"],
Cell[20912, 705, 72, 1, 27, "Input"],
Cell[20987, 708, 211, 7, 32, "Text",
  Evaluatable->False],
Cell[21201, 717, 66, 1, 27, "Input"],
Cell[21270, 720, 773, 27, 46, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[22080, 752, 106, 2, 46, "Subsection",
  Evaluatable->False],
Cell[22189, 756, 1026, 36, 64, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[23240, 796, 118, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[23361, 800, 491, 16, 117, "Input"],
Cell[23855, 818, 72, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23964, 824, 102, 2, 42, "Subsubsection",
  Evaluatable->False],
Cell[24069, 828, 325, 8, 132, "Input"],
Cell[24397, 838, 72, 1, 27, "Input"],
Cell[24472, 841, 891, 29, 62, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[25412, 876, 110, 2, 46, "Subsection",
  Evaluatable->False],
Cell[25525, 880, 1998, 68, 94, "Text",
  Evaluatable->False],
Cell[27526, 950, 1447, 37, 357, "Input"],
Cell[28976, 989, 83, 1, 27, "Input"],
Cell[29062, 992, 1079, 32, 78, "Text",
  Evaluatable->False],
Cell[30144, 1026, 546, 9, 237, "Input"],
Cell[30693, 1037, 84, 1, 27, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

