(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     33197,       1134]*)
(*NotebookOutlinePosition[     34280,       1169]*)
(*  CellTagsIndexPosition[     34236,       1165]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData["Chapter VIII.  Technical Programming Concepts"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[" 21. Function Calls", "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision:  April 15 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In this section we consider in detail the procedure a compiled program \
uses in executing a function call, especially the trade-offs of \
call-by-value, call-by-name and call-by-reference, and we describe how these \
methods are simulated in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  Our examples are derived from list and matrix operations."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" operations/structures introduced: \n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["     ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["call-by-value\n     call-by-name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["     call-by-reference\n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["     SetAttributes[",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["attribute",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["]\n     ClearAttributes[",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["attribute",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["]\n     Attributes[",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[
  "]\n     HoldAll, HoldFirst, HoldRest\n     AppendTo[]\n     PrependTo[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Function Calls"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Recall how standard function calls in compiled programs operate.  The \
computer copies the values of the actual parameters into the space allocated \
for the formal parameters; the computer runs the function, which is typically \
stored in a separate place from the list of instructions being executed; and \
finally the computer optionally provides a return value back to the original \
program.  This method of implementing function calls is termed ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["call-by-value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[", emphasizing the fact that only the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" of each parameter is passed to the function.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"To illustrate this, consider the following version of an absolute value \
function."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Clear[absValue];",
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox["\nabsValue[x_] := If[x < 0, -x, x];",
    AspectRatioFixed->True]
}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Tracing an evaluation of this function reveals something about how ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" deals with parameters when evaluating functions.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["n = 5; \nTrace[absValue[2n]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The steps revealed by the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" are: \n  \[Bullet]  Compute the value of the argument, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", by finding ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["'s value, then doubling (to get ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["10",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[");\n  \[Bullet]  Substitute ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["10",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" for ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["every",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" instance of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" in the body of the function ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["absValue",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ";\n  \[Bullet]  Proceed to evaluate the body of the function.\nThe second \
step is the reason ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" doesn't allow you to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Set",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " parameters to new values within the body of a function.  Here, for \
example, a statement like",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" x = -x ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["(or, in FullForm,  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Set[x, -x]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[") would be converted to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" 10 = -10 ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["(or, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Set[10, -10]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[").  Clearly this is ridiculous.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Example: matrixTrace"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "There are times, however, when evaluation of the argument to a function \
involves more work than is necessary.  One such situation is when the \
argument is an object of very large size, such as an ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1000x1000",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " array of integers.  The mere act of copying the value of this argument \
into one place in the body of the function involves copying one million \
integers!  But suppose our function doesn't need to look at all the entries, \
but only at a relatively small fraction of them.  \n\nHere's an example.  The \
trace of a square matrix is the sum of the entries on it's diagonal.  Here's \
a function which computes the sum of the entries on the main diagonal of a \
square matrix.  (In linear algebra, this is known as the \"trace\" of a \
matrix.  First, we write a boolean function which tests for a square \
matrix.)",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[" ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox["squareMatrixQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSize->9],
  StyleBox["...",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSize->9]
}], "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True,
  FontSize->9],

Cell[TextData[{
  StyleBox["Clear[squareMatrixQ];\n",
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox[
  "squareMatrixQ::usage=\"squareMatrixQ[m] returns True if m has dimensions \
{n,n} for some n, otherwise squareMatrixQ[m] returns false.\";\n",
    AspectRatioFixed->True,
    FontSize->9,
    FontColor->RGBColor[1, 0, 1]],
  StyleBox[
  "\nsquareMatrixQ[m_] :=\n  MatrixQ[m] &&\n  Switch[Dimensions[m],\n    \
{n_Integer,n_Integer, ___}, True,\n    _, False\n  ];",
    AspectRatioFixed->True]
}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Clear[matrixTrace];\n",
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox[
  "matrixTrace::usage=\"matrixTrace[m] returns sum of the entries on the\n\
main diagonal of the square matrix m.\";\n",
    AspectRatioFixed->True,
    FontSize->9,
    FontColor->RGBColor[1, 0, 1]],
  StyleBox[
  "\nmatrixTrace[mtrx_?squareMatrixQ] := \n  Module[\n    { diagSum = 0, i\n  \
  },\n    Do[ \n      diagSum = diagSum + mtrx[[i,i]],\n      {i, \
Length[mtrx]}\n    ];\n    diagSum\n  ];",
    AspectRatioFixed->True]
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["matrixTrace[{{10,2,3},{4,50,6},{7,8,90}}]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A call like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace[{{10,2,3},{4,50,6},{7,8,90}}]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " isn't too bad (try tracing it), but suppose we wanted to calculate the \
trace of a large matrix. The variable ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" below holds a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["400x400",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " matrix whose entries are randomly selected from 1, 2, 3, .., 10. ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"largeMtrx = \n  Table[Random[Integer, {1, 10}], {400}, {400}];"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A call like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace[largeMtrx]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " now requires copying 160,000 integers, but we only need to look at the \
400 (or 0.25%) of these.  Let's see how long it takes to do this.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["Timing[matrixTrace[largeMtrx]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If our task involved taking a lot of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "'s for large matrices, the time could really add up.   One way to get \
around this is to delay the evaluation of the argument sent to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" until it is actually needed.  ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " accomplishes this by allowing us to set an \"attribute\" for our function \
indicating that arguments are to be \"held\" unevaluated until it is \
necessary to do so.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Clear[matrixTraceH];\n",
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox[
  "matrixTraceH::usage=\"matrixTraceH[m] returns sum of the entries on the\n\
main diagonal of the square matrix m.\";\n",
    AspectRatioFixed->True,
    FontSize->9,
    FontColor->RGBColor[1, 0, 1]],
  StyleBox[
  "\nSetAttributes[matrixTraceH, HoldAll];\n\nmatrixTraceH[mtrx_Symbol] := \n \
 Module[\n    { diagSum = 0, i\n    },\n    If[squareMatrixQ[mtrx],\n      \
Do[ \n        diagSum = diagSum + mtrx[[i,i]],\n        {i, Length[mtrx]}\n   \
   ];\n      diagSum\n    ]\n  ];",
    AspectRatioFixed->True]
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["Timing[matrixTraceH[largeMtrx]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Timing[squareMatrixQ[largeMtrx]]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Attributes",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " are properties of functions which affect how they are evaluated.  They \
are turned on or off via the commands ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["SetAttributes",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["ClearAttributes",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", respectively.  You can learn the attributes associated with a function \
using the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Attributes",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" command.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["?SetAttributes"], "Input",
  AspectRatioFixed->True],

Cell[TextData["?ClearAttributes"], "Input",
  AspectRatioFixed->True],

Cell[TextData["?Attributes"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Attributes[Plus]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Attributes[matrixTrace]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["Attributes[matrixTraceH]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["There are many attributes available to the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " programmer; our concern here is with those which \"hold\" arguments: ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldAll",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldFirst",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldRest",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  (See the statement ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["SetAttributes[matrixTraceH, HoldAll]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" above.)  We take advantage of \"holding\" when we ask for a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Trace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" of an expression's evaluation.  See, for example, the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["FullForm",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" of the trace in the absolute value example.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"n = 5; \nTrace[absValue[2n]]\n\nTrace[absValue[2n]]//FullForm"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The method of function evaluation that we establish via ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["SetAttributes[matrixTraceH, HoldAll]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" is known as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["call-by-name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[
  ".  This leaves the actual parameters unevaluated until they are needed, \
but evaluates them anew with each reference. ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Warning: the conveniences of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["call-by-name",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[
  " must be balanced against the potential abuse by harmful side-effects.  \
More on this later.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Call-by-reference"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Most commonly used languages (for example, Pascal, C, or C++) do not \
directly support call-by-name function evaluation.  There, the alternative \
mechanism to call-by-value evaluation is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["call-by-reference",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox[" evaluation.  A ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["reference",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " to a variable is the information the computer needs to determine the \
precise storage location of the actual parameter.  You could think of it as \
the location (or ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["address",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ") of the container for this variable---where the computer looks to find \
the value of the variable.  One possible trade-off is that the function now \
has the information, and therefore the ability, to change the value of the \
original actual parameter, so that if the function changes the value of the \
formal parameter, it will then change the value of the original actual \
parameter.  This behavior may or may not be one we wish the function to have. \
",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" were implemented using call-by-reference, then a call like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace[largeMtrx]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" would make the address of the parameter ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["mtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" (in the definition of the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[") agree with the address of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  Then the computations referencing parts of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["mtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" would be going to the actual source, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", to get the needed values, as opposed to retrieving these values from a \
copy of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[". Since it's only necessary to copy the address of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and not the entire value of ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", considerable time savings are involved.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "An inconvenience of call-by-reference evaluation is that calls to a \
function must provide an actual parameter which has a \"real\" address.  For \
example, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTrace[2 * largeMtrx]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" wouldn't be permitted, since the expression ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2*largeMtrx",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " doesn't belong to a \"name\"---we haven't specifically asked for an \
address to store this value.  We imposed a similar restriction on ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matrixTraceH",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", when the formal parameter was typed as a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Symbol",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Call-by-reference is an important and common tool.  Our discussion of \
call-by-name evaluation is prompted by a desire to simulate the benefits (and \
pitfalls) of call-by-reference evaluation.   "], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Example: saveRotateLeft"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Here we implement a rotate function using call-by-name, and the function \
will have a property that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["no",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " other function we have written so far has had: the capability to save the \
result back in the variable whose name was passed to the function.  We must \
be careful, then, since if the parameter is not a name (but an explicit \
expression, such as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\"), we do not want to allow the function to attempt to change that value. \
 Actually, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " won't allow it, but will complain with an error message, which we'd like \
to avoid.  Hence we check for a name (a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Symbol",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[") in the formal parameter.  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Clear[saveRotateLeft];\n",
    AspectRatioFixed->True,
    FontSize->9],
  StyleBox[
  "\nSetAttributes[saveRotateLeft, HoldAll];\n\nsaveRotateLeft[v_Symbol] := \n\
  Module[\n    {firstEntry},\n    firstEntry = v[[1]];\n    Do[\n      v[[i]] \
= v[[i+1]],\n      {i,1,Length[v]-1}\n    ];\n    v[[ Length[v] ]] = \
firstEntry;\n    v\n  ]",
    AspectRatioFixed->True]
}], "Input",
  AspectRatioFixed->True],

Cell[TextData["test = Range[10]"], "Input",
  AspectRatioFixed->True],

Cell[TextData["saveRotateLeft[test]"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Now ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["test",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" has actually been changed!",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["test"], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["If we were able to use call-by-reference, function calls to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["saveRotateLeft[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " would run faster.  (This isn't really the case with call-by-name in this \
instance, since we need to reference the entire parameter, anyway.)  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "However, it may not be true that we wish our functions to have the power \
to change the arguments which they are given.  We have, then, a time versus \
protection trade-off.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["saveRotateLeft[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is a substitute for \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["RotateLeft[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", but not a substitute without disadvantages:  if we want to rotate a \
list and save the result elsewhere, we would have to say \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["list = listold; saveRotateLeft[list]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", thereby saving the list before \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["saveRotateLeft[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" sank its claws into the old list!",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Notes"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["When a function is defined with the attribute ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldAll",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", the parameter type checking of the function is basically disabled; ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" will only be able to tell if the parameter is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Symbol",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" or not.  Hence, do not check for a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["List",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " or some other type when specifying arguments; do not use expressions such \
as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a_List",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["It is possible in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " to permit only the first argument in a list to be passed under \
call-by-name; one sets the attribute \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldFirst",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" instead of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["HoldAll",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" itself actually contains some call-by-name functions: two are ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["AppendTo[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["PrependTo[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{626, 560},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003809T?o>old</PW30fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@5caU0000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 120, 2, 136, "Title",
  Evaluatable->False],
Cell[1854, 55, 84, 2, 98, "Title",
  Evaluatable->False],
Cell[1941, 59, 98, 2, 26, "SmallText",
  Evaluatable->False],
Cell[2042, 63, 441, 10, 66, "Text",
  Evaluatable->False],
Cell[2486, 75, 2121, 78, 152, "SmallText",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[4632, 157, 91, 2, 50, "Section",
  Evaluatable->False],
Cell[4726, 161, 1035, 26, 96, "Text",
  Evaluatable->False],
Cell[5764, 189, 158, 4, 30, "Text",
  Evaluatable->False],
Cell[5925, 195, 209, 7, 39, "Input"],
Cell[6137, 204, 433, 14, 48, "Text",
  Evaluatable->False],
Cell[6573, 220, 81, 1, 42, "Input"],
Cell[6657, 223, 2999, 108, 134, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[9681, 335, 100, 2, 46, "Subsection",
  Evaluatable->False],
Cell[9784, 339, 1135, 25, 116, "Text",
  Evaluatable->False],
Cell[10922, 366, 393, 17, 22, "SmallText",
  Evaluatable->False],
Cell[11318, 385, 546, 15, 110, "Input"],
Cell[11867, 402, 580, 16, 150, "Input"],
Cell[12450, 420, 94, 1, 22, "Input"],
Cell[12547, 423, 899, 30, 51, "Text",
  Evaluatable->False],
Cell[13449, 455, 116, 2, 30, "Input"],
Cell[13568, 459, 483, 15, 38, "Text",
  Evaluatable->False],
Cell[14054, 476, 83, 1, 22, "Input"],
Cell[14140, 479, 1038, 32, 64, "Text",
  Evaluatable->False],
Cell[15181, 513, 665, 17, 190, "Input"],
Cell[15849, 532, 84, 1, 22, "Input"],
Cell[15936, 535, 85, 1, 22, "Input"],
Cell[16024, 538, 989, 35, 51, "Text",
  Evaluatable->False],
Cell[17016, 575, 67, 1, 22, "Input"],
Cell[17086, 578, 69, 1, 22, "Input"],
Cell[17158, 581, 64, 1, 22, "Input"],
Cell[17225, 584, 69, 1, 22, "Input"],
Cell[17297, 587, 76, 1, 22, "Input"],
Cell[17376, 590, 77, 1, 22, "Input"],
Cell[17456, 593, 1667, 56, 64, "Text",
  Evaluatable->False],
Cell[19126, 651, 115, 2, 50, "Input"],
Cell[19244, 655, 721, 22, 51, "Text",
  Evaluatable->False],
Cell[19968, 679, 456, 15, 38, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[20461, 699, 97, 2, 30, "Subsection",
  Evaluatable->False],
Cell[20561, 703, 1515, 40, 103, "Text",
  Evaluatable->False],
Cell[22079, 745, 2322, 78, 77, "Text",
  Evaluatable->False],
Cell[24404, 825, 1239, 39, 64, "Text",
  Evaluatable->False],
Cell[25646, 866, 272, 5, 38, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[25955, 876, 103, 2, 30, "Subsection",
  Evaluatable->False],
Cell[26061, 880, 1406, 44, 77, "Text",
  Evaluatable->False],
Cell[27470, 926, 436, 11, 160, "Input"],
Cell[27909, 939, 69, 1, 22, "Input"],
Cell[27981, 942, 73, 1, 22, "Input"],
Cell[28057, 945, 340, 13, 25, "Text",
  Evaluatable->False],
Cell[28400, 960, 57, 1, 22, "Input"],
Cell[28460, 963, 529, 15, 38, "Text",
  Evaluatable->False],
Cell[28992, 980, 1289, 39, 77, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[30318, 1024, 85, 2, 30, "Subsection",
  Evaluatable->False],
Cell[30406, 1028, 1280, 44, 51, "Text",
  Evaluatable->False],
Cell[31689, 1074, 801, 29, 38, "Text",
  Evaluatable->False],
Cell[32493, 1105, 664, 24, 25, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

