(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     72427,       2426]*)
(*NotebookOutlinePosition[     73513,       2461]*)
(*  CellTagsIndexPosition[     73469,       2457]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[" 26. The C Language"], "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Last revision:  April 15 1998", "SmallText",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "This section introduces a subset of the C language and explores its \
similarities and differences with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["A Quick Tour of the C Language"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The C language is a computer programming language widely used by \
professional programmers, and for that reason we believe that students should \
have some exposure to the language.  While structuring an introductory \
computing class around C is possible, it poses pedagogical burdens which we \
did not wish to assume, one of the most important being that it initially \
takes students longer to write programs in C than in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", and another being that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " contains many more built-in functions which C, in its original state, \
does not support.  However, C shares so many similarities to certain aspects \
of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " that we can introduce C by highlighting these similarities.  This section \
is by no means intended as a tutorial which will result in the reader's \
immediately being able to write perfect C code; instead, it is meant to serve \
as a reader's guide to literature in C.  The student who wishes to program \
professionally should take at least a second course in programming structured \
around C or some other common language."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Data Types, Declarations, and Definitions"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C is a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["functional ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["language, just like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", which means that it is primarily structured around variables and \
functions which act upon them.  Unlike ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[", however, C requires that ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["every",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " variable and function have a data type, and these types cannot change \
over the course of the program.  For instance, if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" is an integer, it must always denote an integer, and if an approximate \
number is assigned to it, \"i\" will hold only the integer portion.  What we \
mean when we say \"the data type of a function\" is really the data type of \
the return value, and if a function does not return a value, its data type \
(or \"return type\", as we will use later) must defined as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["void",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["What data types does C support, other than \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["void",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" for functions with no return value?  Here is a partial list:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["char",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" (values must be single characters); \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["int",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (values must be machine-precision integers, typically ranging in value \
from ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-32767",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["+32767",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["); \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["long",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (values are integers which take up several bytes, so typically ranging \
from ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-2147483647",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2147483647",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["); \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["float",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (values are machine-size approximate numbers, typically ranging from ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-10^37",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["10^37",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  ", with up to 6 significant digits); \"double\" (values are approximate \
numbers allowed more bytes than floats, so with a wider range of values and \
with more significant digits); and arrays of any of these types.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that C does ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["not ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  "support lists or strings as separate data types.  Instead, C programmers \
must keep arrays of integers or characters and must worry about the lengths \
of these arrays themselves.  While ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " allows lists of arbitrary length and numbers of arbitrary size, C instead \
restricts the values to machine-precision numbers.  In C, to use huge \
integers, it is necessary to store them in pieces and manipulate their \
separate parts (digits, say) separately.  C is thus a more primitive language \
than ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ".   However, one can write packages so that C can use arbitrary-length \
lists and numbers, if one desires, and C's chief advantages over ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " include its versatility and its compiled, rather than interpreted, \
implementation.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["Declaring Variables and Functions; Arrays"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Again, every variable and function in C must have its type declared.  The \
syntax for declaring variables and functions is to write, on a separate line, \
the data type name, followed by a space, followed by the names of the \
variables and functions which have that type.  Function names must be \
suffixed with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", which indicates that the name refers to a function, not a variable, \
and if the function takes any arguments, they must be included between the \
parentheses, separated by commas, each preceded by its data type.  In order \
to declare a variable to be an array of elements of a certain type, the \
variable name must be suffixed with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["[number]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", where \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is an integer, in which case an array of size \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["number",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" will be declared.  Finally, all C lines must end with a semicolon.  \
Here are some examples:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int a,b,c;\nfloat sin(),myFunc(float b);\nchar str[30];"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["In this example we have three integer variables defined, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"; two functions which return float values, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["sin",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["myFunc",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", the first of which takes no arguments and the second of which takes \
one argument of type float; and finally an array of 30 characters, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["str",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Arrays in C deserve special mention, because in C the first array element \
is referenced with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["[0]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and the last element (in an array with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" elements) \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["[n-1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Hence, the values in a three-element array \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" are \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a[0]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a[1]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a[2]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Also, arrays with more than one dimension are declared simply by \
suffixing more \"[number]\" components:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["int a[10][10]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" declares a 10 by 10 array of integers.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Several syntactical differences with ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" are immediately apparent.  First, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " suffixes functions with square brackets which enclose the arguments, if \
any; C instead uses parentheses.  Also, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " uses double brackets to denote an array, but C uses single square \
brackets.  Finally, ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " does not permit the declaration of an array without constructing it, as \
with a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Table[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"Defining Variables and Functions; Return Statements; Local, Global"], 
  "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["In C, just as in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", variables are defined or modified with assignment statements, as in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i=3;",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Functions are defined similarly to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", as follows.  The return type of the function is specified first, \
followed by the name of the function, followed by a parenthesized, \
comma-separated sequence of argument names, each preceded by the appropriate \
data type, followed by a curly-brace-enclosed set of statements inside the \
function, called the \"function block\".  The statements inside a C \
\"function block\" are similar to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " statements, as we will see, later; however, we point out now that the \
return value must be contained in a statement \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["return x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", where \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["x",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" is the value the function wishes to return---the return value is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["not",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" simply the value of the last statement executed, as it is in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here is an example of definitions of a variable \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and a function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["myFunc",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", which multiplies its first argument, an integer, by 2 and adds the \
second argument, returning the result.  (The variable definition here has \
nothing to do with the function definition but is provided only as an \
example.)",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"i = 4;  \n\nint myFunc(int a, int b) {\n   int c;\n   c = 2*a + b;\n   \
return c;\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Notice that a separate variable was defined inside the function block.  \
This variable, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", will be a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["local variable",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" in C because it is declared inside the function block. ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" Global variables",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  ", on the other hand, are those variables declared outside any function \
block.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "One difference with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is that, in C, when a return statement is encountered, the function \
immediately stops and returns the value; no succeeding statements will be \
executed.  For instance, in the function"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int myFunc2(int a);\n\nint myFunc2(int a) {\n\t  return 1;\n\t  return a;\n\
}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"a \"1\" will always be returned, and the statement \"return a\" would never \
be executed."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Control Structures"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "C's control structures are also very similar to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s.  We'll sample the most important ones here."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["If"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C's \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["If",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" statement has the following syntax:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["if (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["boolean-expression",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[") ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", and the statement is executed only if the boolean condition is true.  \
When several statements are to be executed, they should be listed, separated \
by semicolons as usual, with the entire group surrounded by curly braces to \
form a block.  For example,",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"if (a==1) b = 2;\nb = 3;\nif (a==2) { b = 3; c = 4; }"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["causes \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b=2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to be executed only if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", after which \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is set equal to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", and then, if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[", then \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is set to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to 4.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "If we wish to have other statements executed if the boolean value is \
false, the syntax is \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["if (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["boolean-expression",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[") ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; else ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", where again a statement can be replaced by a curly-brace-enclosed \
block.  Hence we could have the following recursive function to compute \
factorials (notice that the function is declared before it it defined!):",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int myFac(int i);\n\nint myFac(int i) {\n\t  if (i==1) return 1;\n\t  else \
return i*myFac(i-1);\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Switch, Break"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C's switch statement has the following standard syntax:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["switch (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["variable",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[") { case ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[": ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; case ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[": ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; ....}",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", where as many \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["case ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[": ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" structures many be repeated as necessary.  The value of the variable \
will be checked, the program will jump to the statement following \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["case ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["value",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[
  "\" with the same value, and will then continue down through the rest of \
the statements until a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["break;",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" statement is encountered, in which case the switch statement will \
terminate.  As usual, one may substitute a block of statements enclosed by \
curly braces for a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\" in the syntax expression.  Here's one example.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void commentOnGrade(char c);\n\nvoid commentOnGrade(char c){\n\t  switch (c) \
{\n\t  \t  case 'A' : { printf(\"Excellent!\\n\"); break;}\n\t  \t  case 'B' \
: { printf(\"Good!\\n\"); break;}\n\t  \t  case 'C' : { printf(\"Fair.\\n\"); \
break;}\n\t  \t  case 'D' : { printf(\"Disappointing\\n\"); break;}\n\t  \t  \
case 'F' : { printf(\"!\\n\"); break;}\n\t  }\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Note that without the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["break",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" statements, each statement after one jumped to by the case would have \
been executed, so that \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["commentOnGrade('A')",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" would have caused all five print statements to be executed.  Notice \
also that single characters are surrounded by single quotes---in essence, \
they are separate from strings---and the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["printf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" command behaves much like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["'s \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Print[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" command.  Finally, notice that there is no \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["return",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" statement, since the function has a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["void",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" return value.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["For"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C's For statement is very similar to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["'s; the only difference is that the first three statements in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  "'s For command are separated by commas and enclosed by parentheses, and \
the fourth statement simply follows the parentheses, as in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["for (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["initialization-statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["boolean-expression",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["; ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["increment-statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[") ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["execution-statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  Here is an example.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int i,j;\n\nj = 0;\nfor (i=1; i<10; i++)\n   j = j + i;"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"As might be expected, this section of code adds up the numbers from 1 to \
9."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["While"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C's While statement is also very similar to ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["'s, having syntax \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["while (",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["boolean-expression",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[") ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["statement",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[";",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  An example is then",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int countDigits(int a);\n\nint countDigits(int a) {\n\t   int b,c;\n\t   b = \
1;\n\t   c = 10;\n\t   while (c<a) {\n\t   \t  b++;\n\t   \t  c*=10;\n\t   }\n\
\t   return b;\n}\n\t"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Do...While"], "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C's ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" statement, however, differs from ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["'s, because C's ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " statement is simply a While statement with the boolean test at the end \
instead of the beginning.  In other words, the execution is the same as a \
While loop except that the loop is executed once before the boolean test is \
checked.  C's ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Do",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " statement, therefore, does not permit the use of iterators.  Here is an \
example which counts the number of zeros in an array which is a global \
variable.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"int a[100];\nint numZeros();\n\nint numZeros() {\n\t\t\tint i,j;\n\t\t\tj = \
0;\n\t\t\ti = 0;\n\t\t\tdo {\n\t\t\t   if (a[i]==0) j++;\n\t\t\t   i++;\n\t\t\
\t} while (i<100);\n\t\t\treturn j;\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["main()"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "When one runs a C program, the first function executed is always the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["main()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" function.  Hence, every C program must have a \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["main",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" function defined somewhere, which marks the entry point into the \
program.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["main()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" typically has void return type.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[
"File Access Basics; Reading, Writing; Standard Input, Output"], "Subsection",\

  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "C uses streams for file management, just like ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  For reasons we won't go into here, a stream variable must be preceded \
by an asterisk in its declaration:"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["FILE *fp;"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Apart from that peculiar detail, however, the open, close, and \
string-printing functions are quite similar:"], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"fp = fopen(\"myfile\",\"w\")  /* w is for writing */\nfprintf(fp,\"First \
line of the file\\n\");\nfclose(fp);"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "In order to read or write variables, however, C must be told what sort of \
format the values will be in.  For reading from a file, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fscanf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (which stands for \"file-scan-formatted\") must first be told the \
stream variable (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fp",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" above), followed by a string with symbols indicating what is to be read \
in---\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%d",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" means an integer, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%f",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" an approximate number, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%c\"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  " a character, and so forth---followed by the variable names to store the \
values in, each preceded by an ampersand (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\").  For writing to a file, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fprintf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (which stands for \"file-print-formatted\") must be told the stream \
variable (\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fp",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\") above, followed by a string with symbols indicating what it to be \
printed---characters interspersed with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%d",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" for integral output, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%f\"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[" for approximate numeric output, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["%c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" for single-character output, and so on---followed by the variable names \
whose values need to be written out.  These strings are the \"formatting\" of \
the input and the output referred to in the names of the functions.  Here are \
a couple of examples:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["int i;\nfprintf(fp,\"The answer is %d\\n\",i);"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "The code above prints a single line with the numeric value of \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["i",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" after the string",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[" \"The answer is \"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData["char c1,c2,c3;\nfscanf(fp,\"%c%c%c\",&c1,&c2,&c3);"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Here, three characters are read from the file and placed in variables \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c2",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c3",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", respectively.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Finally, two files are always associated to a C program, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdin",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdout",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  The former, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdin",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", is the \"standard input\" file, which typically means input from the \
keyboard.  The latter, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdout",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", is the \"standard output\" file, typically meaning the console screen. \
 These two stream names \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdin",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdout",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" can be used with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fscanf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["fprintf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", or they can be omitted if one uses the functions \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["printf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["scanf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\"---\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["printf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" writes to the \"standard output\" file, while \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["scanf",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" reads from the \"standard input\" file.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Comments, Operators, Libraries"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["In C comments are enclosed not by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["(*",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*)",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" as in ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[", but by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["/*",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*/",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\". ",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Many of the operators of C and ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" are identical.  Here are the most important:  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["!",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" (not), \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["+",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["/",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["==",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["!=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["<",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[">",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["<=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox[">=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["++",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["--",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["+=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["-=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["*=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["/=",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["||",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["&&",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["\"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" (quotation mark to mark beginning or ending of a string), and \
parentheses to show precedence in an expression.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Finally, just as ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " has separate packages which can be loaded to permit greater \
functionality, C programs can include functions from other libraries which \
add to the functions the program can use.  These are usually indicated at the \
top of the program with \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["#include <",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["filename",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[">",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" or \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["#include \"",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["filename",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox["\" ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" statements.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["Reading Examples"], "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"In this section we will present entire C programs and comment on their \
functionality."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData["1: primeTester"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["C does not have a prime-tester function like ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox["'s \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["PrimeQ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Here we present a program which would allow a user to enter a \
machine-size integer and have its primality determined.  We use the fact that \
if an integer has no divisor smaller than its square root, it must be prime.  \
Note that when one divides two integers in C, the result is the integer \
quotient---the remainder is thrown away.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void main() {\n\t  int a,b,c,d;\n\t  \n\t  printf(\"Please enter an integer.\
\\n\");\n\t  scanf(\"%d\",&a);\n\t  b = sqrt(a);\n\t  d = 0;\n\t  for (c=2; \
c<=b; c++) \n\t     if (((a/c)*c)==a) d = 1;  /* evenly divides! */\n\t  if \
(d==1) \n\t     printf(\"Not prime!\\n\");\n\t  else\n\t  \t  printf(\"Prime!\
\\n\");\n}  \n\t    "], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here no \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["#include",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" statements were necessary, since we did not use any non-standard C \
functions. The program writes out the query line, reads in a number from the \
keyboard, and sets \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to be its square root (rounded to the next lowest integer, since the \
result is placed in \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", an integer).  Then, one at a time, it decides if any of the numbers \
between 2 and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["b",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" evenly divide \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", by taking the integer quotient, multiplying back by \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", and seeing if the result equals \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", which would only occur if \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" evenly divided \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["a",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  If one does evenly divide, then a flag \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["d",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" is set to \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["1",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\", indicating that the number is not prime.  A message indicating the \
primality is finally written out to the screen.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["2: factorialComputer"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Here we present a program which writes out \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" factorial, as \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["n",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" runs from 1 to 7, to a file \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["myFile",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\", one number to a line.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"#include <stdio.h>\n\nint fac(int a);\n\nint fac(int a) {\n\t  if (a<=1) \
return 1;\n\t  else return a*fac(a-1);\n}\n\nvoid main() {\n   FILE *fp;\n   \
int a;\n   \n   fp = fopen(\"myFile\",\"w\");\n   for (a=1; a<=7; a++)\n      \
fprintf(fp,\"%d\\n\",fac(a));\n   fclose(fp);\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Here we needed to include the standard input/output function library, \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["stdio.h",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\".  Note that we could not have increased the number of factorials to 8 \
without changing the data types of the factorial function and its arguments, \
since 8! cannot be stored in a machine-size integer.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData["3: grades"], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["We end with a version of the \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Grades`",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" package defined in a previous section, converted to be a C program.  In \
order to implement the range of choices permitted by the package, we \
introduce a menu function which writes a list of options to the screen, one \
of which the user must indicate by entering appropriate data from the \
keyboard.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"#include <stdio.h>\n\nfloat grades[100];  /* global variable to hold up to \
100\n\t\t\t\t\t\t                 grades */\nint numGrades;      /* global \
variable to indicate how many\n                       grades actually stored \
*/\n                       \nvoid addGrade(float g),deleteGrade(float g);\n\
void listGrades(),gradeInit(),printMenu();\nfloat findAverage();\n\nvoid \
gradeInit() {\n\t  int i;\n\t  for (i=0; i<100; i++)\n\t     grades[i] = 0.0; \
   /* unnecessary but nice to\n\t                            clear array */\n\
\t  numGrades = 0;\n}\n\nvoid addGrade(float g){\n   if (numGrades>=100)\n    \
  printf(\"Full of grades!  No more allowed!\\n\");\n   else {\n     \
grades[numGrades] = g;\n     numGrades++;\n   }\n}\n\nvoid deleteGrade(float \
g){\n   int a,b,c;\n   b = -1;\t                /* -1 is a flag for 'not \
found' */\n   for (a=0; a<100; a++)\n      if (b==-1)           /* if not \
found, try */\n         if (grades[a]==g) b=a;  /* b has the index of g */\n  \
 if (b==-1) \n      printf(\"Grade %f not found!\\n\",g);\n   else {\n      \
for (c=b; c<numGrades-1; c++)   /* move all \n                                \
         grades down */\n         grades[c] = grades[c+1];\n      \
numGrades--;\n   }\n}\n\nvoid listGrades() {\n\t\t\tint i;\n\t\t\tfor (i=0; \
i<numGrades; i++)\n\t\t\t   printf(\"Grade number %d : %f\\n\",i,grades[i]);\n\
}\n\nfloat findAverage(){\n\t\t\tfloat sum,ii;\n\t\t\tint i;\n\t\t\tsum = \
0.0;\n\t\t\tfor (i=0; i<numGrades; i++)\n\t\t\t   sum += grades[i];\n\t\t\tii \
= numGrades;        /* converts numGrades to float */\n\t\t\tif (numGrades<1) \
\n\t\t\t     sum = 0;\n\t\t\telse \n\t\t\t     sum = sum / ii;\n\t\t\treturn \
sum;\n}\n\nvoid printMenu() {\n    printf(\"Enter one of the following \
choices.\\n\");\n    printf(\"I: initialize grades\\n\");\n    printf(\"A: \
add a grade\\n\");\n    printf(\"D: delete a grade\\n\");\n    printf(\"L: \
list grades\\n\");\n    printf(\"F: find average\\n\");\n    printf(\"Q: quit\
\\n\");\n}\n\nvoid main() {\n\t   char c,c2;\n\t   float g;\n\t   \
gradeInit();\n\t   c = ' ';\n\t   while (c!='Q') {\n\t        do {\n\t        \
    printMenu();\n\t            scanf(\"%c\",&c);\n\t            \
scanf(\"%c\",&c2);    /* take return key out\n\t        \t\t\t\t\t\t          \
           of input stream */\n\t        } while ( (c!='I') && (c!='A') && \
(c!='D') &&\n\t        \t\t        (c!='L') && (c!='F') && (c!='Q') );\n\t    \
    if (c=='A'||c=='D') {\n\t            printf(\"Grade?\\n\");\n\t           \
 scanf(\"%f\",&g);\n\t\t\t          scanf(\"%c\",&c2);    /* same thing */\n\t\
\t      \t}\n\t\t\t     switch (c) {\n\t\t\t          case 'I': {gradeInit(); \
break;}\n\t\t\t          case 'A': {addGrade(g); break;}\n\t\t\t          \
case 'D': {deleteGrade(g); break;}\n\t\t\t          case 'L': {listGrades(); \
break;}\n\t\t\t          case 'F': {printf(\"Average %f\\n\",\n\t\t\t         \
      findAverage()); break;}\n\t\t\t     }\n    }\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"Let's look through this C program piece by piece.  The matter at the top \
reads in the appropriate input/output libraries of commands and declares \
several functions and data."], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"#include <stdio.h>\n\nfloat grades[100];  /* global variable to hold up to \
100\n\t\t\t\t\t\t                 grades */\nint numGrades;      /* global \
variable to indicate how many\n                       grades actually stored \
*/\n                       \nvoid addGrade(float g),deleteGrade(float g);\n\
void listGrades(),gradeInit(),printMenu();\nfloat findAverage();"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["gradeInit()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" sets \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["numGrades",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to 0 and zeros out the array \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["grades",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void gradeInit() {\n\t  int i;\n\t  for (i=0; i<100; i++)\n\t     grades[i] \
= 0.0;    /* unnecessary but nice to\n\t                            clear \
array */\n\t  numGrades = 0;\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["The function \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["addGrade()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" takes an argument, and, assuming there is room in the array \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["grades",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" to add an additional grade to it, does so, incrementing \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["numGrades",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" in the process.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void addGrade(float g){\n   if (numGrades>=100)\n      printf(\"Full of \
grades!  No more allowed!\\n\");\n   else {\n     grades[numGrades] = g;\n    \
 numGrades++;\n   }\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["deleteGrade()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" must do more work.  First it looks through the array, attempting to \
find a match with the given argument.  If it does not find one, it complains \
to the operator; otherwise, it eliminates the grade by copying everything in \
the array left one position, beginning with the grade to be deleted.  Hence, \
if the original array contained '1,2,3,4,5', and '3' was deleted, then the \
'4' would be copies to the place of '3', the '5' to the place of '4', \
resulting in '1,2,4,5'.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void deleteGrade(float g){\n   int a,b,c;\n   b = -1;\t                /* -1 \
is a flag for 'not found' */\n   for (a=0; a<100; a++)\n      if (b==-1)      \
     /* if not found, try */\n         if (grades[a]==g) b=a;  /* b has the \
index of g */\n   if (b==-1) \n      printf(\"Grade %f not found!\\n\",g);\n  \
 else {\n      for (c=b; c<numGrades-1; c++)   /* move all \n                 \
                        grades down */\n         grades[c] = grades[c+1];\n   \
   numGrades--;\n   }\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["\"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["listGrades()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" and \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["findAverage()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" are fairly self-explanatory.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["findAverage",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\" must check \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["numGrades",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" to insure that it does not divide by zero, and it returns the average \
as its return value.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void listGrades() {\n\t\t\tint i;\n\t\t\tfor (i=0; i<numGrades; i++)\n\t\t\t \
  printf(\"Grade number %d : %f\\n\",i,grades[i]);\n}\n\nfloat findAverage(){\
\n\t\t\tfloat sum,ii;\n\t\t\tint i;\n\t\t\tsum = 0.0;\n\t\t\tfor (i=0; \
i<numGrades; i++)\n\t\t\t   sum += grades[i];\n\t\t\tii = numGrades;        \
/* converts numGrades to float */\n\t\t\tif (numGrades<1) \n\t\t\t     sum = \
0;\n\t\t\telse \n\t\t\t     sum = sum / ii;\n\t\t\treturn sum;\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox[
  "Finally, we have below the portion of the program which implements the \
outer loop.  \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["main()",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" declares some variables, initializes the grades, and has an outer \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["do...while",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" loop based on the user's command, a character in the variable \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["c",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox["\".  The first \"",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["do",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier"],
  StyleBox[
  "\" loop inside loops until a valid character is entered from the keyboard. \
 Note that in C the input stream contains the return button character, which \
we must take out of the stream in order to process correctly the remaining \
characters.  (Otherwise the return character would be read in as the next \
command, an illegal one, and we would then get the menu printed again.)  If \
the command requires a grade, one is read in, and finally a switch statement \
controls the path of execution.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[
"void printMenu() {\n    printf(\"Enter one of the following choices.\\n\");\n\
    printf(\"I: initialize grades\\n\");\n    printf(\"A: add a grade\\n\");\n\
    printf(\"D: delete a grade\\n\");\n    printf(\"L: list grades\\n\");\n   \
 printf(\"F: find average\\n\");\n    printf(\"Q: quit\\n\");\n}\n\nvoid \
main() {\n\t   char c,c2;\n\t   float g;\n\t   gradeInit();\n\t   c = ' ';\n\t\
   while (c!='Q') {\n\t        do {\n\t            printMenu();\n\t           \
 scanf(\"%c\",&c);\n\t            scanf(\"%c\",&c2);    /* take return key \
out\n\t        \t\t\t\t\t\t                     of input stream */\n\t        \
} while ( (c!='I') && (c!='A') && (c!='D') &&\n\t        \t\t        (c!='L') \
&& (c!='F') && (c!='Q') );\n\t        if (c=='A'||c=='D') {\n\t            \
printf(\"Grade?\\n\");\n\t            scanf(\"%f\",&g);\n\t\t\t          \
scanf(\"%c\",&c2);    /* same thing */\n\t\t      \t}\n\t\t\t     switch (c) \
{\n\t\t\t          case 'I': {gradeInit(); break;}\n\t\t\t          case 'A': \
{addGrade(g); break;}\n\t\t\t          case 'D': {deleteGrade(g); break;}\n\t\
\t\t          case 'L': {listGrades(); break;}\n\t\t\t          case 'F': \
{printf(\"Average %f\\n\",\n\t\t\t               findAverage()); break;}\n\t\t\
\t     }\n    }\n}"], "Print",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{520, 509},
WindowMargins->{{140, Automatic}, {Automatic, 31}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003809T?o>old</PW30fL5:0?l0080004/0B`000000]P2:001
0000I00000400`<300000BL?00400@5caU0000000000060801T1T000000`0000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 94, 2, 98, "Title",
  Evaluatable->False],
Cell[1828, 55, 98, 2, 26, "SmallText",
  Evaluatable->False],
Cell[1929, 59, 243, 8, 48, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2197, 71, 107, 2, 50, "Section",
  Evaluatable->False],
Cell[2307, 75, 1288, 25, 212, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3620, 104, 121, 2, 46, "Subsection",
  Evaluatable->False],
Cell[3744, 108, 1821, 57, 151, "Text",
  Evaluatable->False],
Cell[5568, 167, 2638, 92, 90, "Text",
  Evaluatable->False],
Cell[8209, 261, 1570, 46, 103, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[9804, 311, 124, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[9931, 315, 1641, 46, 103, "Text",
  Evaluatable->False],
Cell[11575, 363, 131, 3, 35, "Print",
  Evaluatable->False],
Cell[11709, 368, 1417, 50, 51, "Text",
  Evaluatable->False],
Cell[13129, 420, 1875, 66, 64, "Text",
  Evaluatable->False],
Cell[15007, 488, 1372, 47, 64, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[16416, 540, 153, 4, 33, "Subsubsection",
  Evaluatable->False],
Cell[16572, 546, 2299, 72, 103, "Text",
  Evaluatable->False],
Cell[18874, 620, 778, 24, 51, "Text",
  Evaluatable->False],
Cell[19655, 646, 161, 4, 71, "Print",
  Evaluatable->False],
Cell[19819, 652, 905, 31, 51, "Text",
  Evaluatable->False],
Cell[20727, 685, 345, 9, 38, "Text",
  Evaluatable->False],
Cell[21075, 696, 154, 4, 62, "Print",
  Evaluatable->False],
Cell[21232, 702, 165, 4, 25, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[21446, 712, 98, 2, 42, "Subsection",
  Evaluatable->False],
Cell[21547, 716, 233, 7, 25, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[21805, 727, 85, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[21893, 731, 1245, 42, 51, "Text",
  Evaluatable->False],
Cell[23141, 775, 129, 3, 35, "Print",
  Evaluatable->False],
Cell[23273, 780, 1922, 76, 38, "Text",
  Evaluatable->False],
Cell[25198, 858, 1318, 45, 51, "Text",
  Evaluatable->False],
Cell[26519, 905, 176, 4, 62, "Print",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[26732, 914, 96, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[26831, 918, 3272, 117, 90, "Text",
  Evaluatable->False],
Cell[30106, 1037, 441, 7, 107, "Print",
  Evaluatable->False],
Cell[30550, 1046, 1800, 60, 77, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[32387, 1111, 86, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[32476, 1115, 1862, 65, 64, "Text",
  Evaluatable->False],
Cell[34341, 1182, 131, 3, 53, "Print",
  Evaluatable->False],
Cell[34475, 1187, 152, 4, 25, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[34664, 1196, 88, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[34755, 1200, 1031, 38, 38, "Text",
  Evaluatable->False],
Cell[35789, 1240, 256, 5, 125, "Print",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[36082, 1250, 93, 2, 33, "Subsubsection",
  Evaluatable->False],
Cell[36178, 1254, 1268, 41, 64, "Text",
  Evaluatable->False],
Cell[37449, 1297, 271, 5, 125, "Print",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37769, 1308, 86, 2, 42, "Subsection",
  Evaluatable->False],
Cell[37858, 1312, 891, 30, 51, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[38786, 1347, 143, 4, 42, "Subsection",
  Evaluatable->False],
Cell[38932, 1353, 293, 8, 38, "Text",
  Evaluatable->False],
Cell[39228, 1363, 84, 2, 20, "Print",
  Evaluatable->False],
Cell[39315, 1367, 184, 4, 25, "Text",
  Evaluatable->False],
Cell[39502, 1373, 187, 4, 35, "Print",
  Evaluatable->False],
Cell[39692, 1379, 3016, 99, 129, "Text",
  Evaluatable->False],
Cell[42711, 1480, 121, 2, 26, "Print",
  Evaluatable->False],
Cell[42835, 1484, 575, 21, 25, "Text",
  Evaluatable->False],
Cell[43413, 1507, 125, 2, 26, "Print",
  Evaluatable->False],
Cell[43541, 1511, 745, 28, 25, "Text",
  Evaluatable->False],
Cell[44289, 1541, 2666, 94, 77, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[46992, 1640, 110, 2, 42, "Subsection",
  Evaluatable->False],
Cell[47105, 1644, 1052, 41, 25, "Text",
  Evaluatable->False],
Cell[48160, 1687, 4095, 162, 51, "Text",
  Evaluatable->False],
Cell[52258, 1851, 1399, 49, 51, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[53706, 1906, 93, 2, 30, "Section",
  Evaluatable->False],
Cell[53802, 1910, 162, 4, 25, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[53989, 1918, 94, 2, 42, "Subsection",
  Evaluatable->False],
Cell[54086, 1922, 879, 25, 64, "Text",
  Evaluatable->False],
Cell[54968, 1949, 409, 7, 143, "Print",
  Evaluatable->False],
Cell[55380, 1958, 2681, 92, 90, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[58098, 2055, 100, 2, 42, "Subsection",
  Evaluatable->False],
Cell[58201, 2059, 761, 27, 25, "Text",
  Evaluatable->False],
Cell[58965, 2088, 358, 6, 170, "Print",
  Evaluatable->False],
Cell[59326, 2096, 596, 17, 51, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[59959, 2118, 89, 2, 42, "Subsection",
  Evaluatable->False],
Cell[60051, 2122, 653, 18, 51, "Text",
  Evaluatable->False],
Cell[60707, 2142, 3066, 41, 926, "Print",
  Evaluatable->False],
Cell[63776, 2185, 252, 5, 38, "Text",
  Evaluatable->False],
Cell[64031, 2192, 450, 7, 98, "Print",
  Evaluatable->False],
Cell[64484, 2201, 719, 27, 25, "Text",
  Evaluatable->False],
Cell[65206, 2230, 261, 5, 71, "Print",
  Evaluatable->False],
Cell[65470, 2237, 818, 28, 38, "Text",
  Evaluatable->False],
Cell[66291, 2267, 252, 5, 80, "Print",
  Evaluatable->False],
Cell[66546, 2274, 805, 20, 64, "Text",
  Evaluatable->False],
Cell[67354, 2296, 577, 9, 143, "Print",
  Evaluatable->False],
Cell[67934, 2307, 999, 36, 38, "Text",
  Evaluatable->False],
Cell[68936, 2345, 529, 8, 179, "Print",
  Evaluatable->False],
Cell[69468, 2355, 1565, 45, 90, "Text",
  Evaluatable->False],
Cell[71036, 2402, 1351, 19, 568, "Print",
  Evaluatable->False]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

